@@ -45,8 +45,8 @@ static const char copyright[] =
 static char sccsid[] = "@(#)ls.c	8.5 (Berkeley) 4/2/94";
 #endif /* not lint */
 #endif
+#if 0
 #include <sys/cdefs.h>
-#ifndef LINUX
 __FBSDID("$FreeBSD: src/bin/ls/ls.c,v 1.72 2002/12/18 21:05:15 rwatson Exp $");
 #endif
 
@@ -70,6 +70,7 @@ __FBSDID("$FreeBSD: src/bin/ls/ls.c,v 1.72 2002/12/18 21:05:15 rwatson Exp $");
 #include <limits.h>
 #include <locale.h>
 #include <pwd.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -95,7 +96,11 @@ __FBSDID("$FreeBSD: src/bin/ls/ls.c,v 1.72 2002/12/18 21:05:15 rwatson Exp $");
 
 static void	 display(FTSENT *, FTSENT *, int);
 static u_quad_t	 makenines(u_long);
+#ifndef NETBSD /* For use with the FreeBSD's implementation of fts functions */
 static int	 mastercmp(const FTSENT * const *, const FTSENT * const *);
+#else /* For use with the original BSD implementation of fts functions */
+static int	 mastercmp(const FTSENT **, const FTSENT **);
+#endif /* NETBSD */
 static void	 traverse(int, char **, int);
 
 static void (*printfcn)(DISPLAY *);
@@ -534,7 +539,10 @@ display(FTSENT *p, FTSENT *list, int options)
 	char *initmax;
 	int entries, needstats;
 	const char *user, *group;
-	char *flags, *labelstr = NULL;
+	char *flags;
+#ifdef HAVE_MAC
+	char *labelstr = NULL;
+#endif
 	char buf[STRBUF_SIZEOF(u_quad_t) + 1];
 	char ngroup[STRBUF_SIZEOF(uid_t) + 1];
 	char nuser[STRBUF_SIZEOF(gid_t) + 1];
@@ -582,13 +590,9 @@ display(FTSENT *p, FTSENT *list, int options)
 			strcpy(initmax2, "0");
 
 		ninitmax = sscanf(jinitmax,
-		    " %lu : %lu : %lu : %i : %i : %i : %llu : %lu : %lu ",
-		    &maxinode, &maxblock, &maxnlink, &maxuser,
-#ifndef LINUX /* BSD source */
-		    &maxgroup, &maxflags, &maxsize, &maxlen, &maxlabelstr);
-#else /* Linux port */
-		    &maxgroup, &maxflags, (long long *)&maxsize, &maxlen, &maxlabelstr);
-#endif /* Linux port */
+		    " %lu : %lu : %lu : %i : %i : %i : %jd : %lu : %lu ",
+		    &maxinode, &maxblock, &maxnlink, &maxuser, &maxgroup,
+		    &maxflags, (intmax_t *)&maxsize, &maxlen, &maxlabelstr);
 		f_notabs = 1;
 		switch (ninitmax) {
 		case 0:
@@ -695,20 +699,17 @@ display(FTSENT *p, FTSENT *list, int options)
 				if ((glen = strlen(group)) > maxgroup)
 					maxgroup = glen;
 				if (f_flags) {
-#ifndef LINUX /* BSD source */
+#ifdef HAVE_FLAGS
 					flags = fflagstostr(sp->st_flags);
 					if (flags != NULL && *flags == '\0') {
 						free(flags);
 						flags = strdup("-");
 					}
+#else /* !HAVE_FLAGS */
+					flags = strdup("-");
+#endif /* HAVE_FLAGS */
 					if (flags == NULL)
 						err(1, "fflagstostr");
-#else /* Linux port */
-					    flags=malloc(2);
-					    if (flags == NULL)
-						err(1, "malloc");
-					    strncpy(flags, "-", 2);
-#endif /* Linux port */
 					flen = strlen(flags);
 					if (flen > (size_t)maxflags)
 						maxflags = flen;
@@ -823,11 +824,7 @@ label_out:
 		d.s_inode = strlen(buf);
 		(void)snprintf(buf, sizeof(buf), "%lu", maxnlink);
 		d.s_nlink = strlen(buf);
-#ifndef LINUX /* BSD source */
-		(void)snprintf(buf, sizeof(buf), "%qu", maxsize);
-#else /* Linux port */
-		(void)snprintf(buf, sizeof(buf), "%qu", (long long)maxsize);
-#endif /* Linux port */
+		(void)snprintf(buf, sizeof(buf), "%jd", (intmax_t)maxsize);
 		d.s_size = strlen(buf);
 		d.s_user = maxuser;
 	}
@@ -846,7 +843,11 @@ label_out:
  * All other levels use the sort function.  Error entries remain unsorted.
  */
 static int
+#ifndef NETBSD /* For use with the FreeBSD's implementation of fts functions */
 mastercmp(const FTSENT * const *a, const FTSENT * const *b)
+#else /* For use with the original BSD implementation of fts functions */
+mastercmp(const FTSENT **a, const FTSENT **b)
+#endif /* NETBSD */
 {
 	int a_info, b_info;
 
