@@ -39,8 +39,8 @@
 static char sccsid[] = "@(#)print.c	8.4 (Berkeley) 4/17/94";
 #endif /* not lint */
 #endif
+#if 0
 #include <sys/cdefs.h>
-#ifndef LINUX
 __FBSDID("$FreeBSD: src/bin/ls/print.c,v 1.64 2002/12/19 01:13:23 tjr Exp $");
 #endif
 
@@ -59,6 +59,7 @@ __FBSDID("$FreeBSD: src/bin/ls/print.c,v 1.64 2002/12/19 01:13:23 tjr Exp $");
 #endif /* Linux port */
 #include <math.h>
 #include <langinfo.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -197,12 +198,8 @@ printlong(DISPLAY *dp)
 		if (f_inode)
 			(void)printf("%*lu ", dp->s_inode, (u_long)sp->st_ino);
 		if (f_size)
-			(void)printf("%*lld ",
-#ifndef LINUX /* BSD source */
-			    dp->s_block, howmany(sp->st_blocks, blocksize));
-#else /* Linux port */
-			    dp->s_block, (long long)howmany(sp->st_blocks, blocksize));
-#endif /* Linux port */
+			(void)printf("%*jd ",
+			    dp->s_block, (intmax_t)howmany(sp->st_blocks, blocksize));
 		strmode(sp->st_mode, buf);
 #ifdef HAVE_ACL
 		/*
@@ -233,12 +230,8 @@ printlong(DISPLAY *dp)
 				(void)printf("%3d, %3d ",
 				    major(sp->st_rdev), minor(sp->st_rdev));
 		else if (dp->bcfile)
-			(void)printf("%*s%*lld ",
-#ifndef LINUX /* BSD source */
-			    8 - dp->s_size, "", dp->s_size, sp->st_size);
-#else /* Linux port */
-			    8 - dp->s_size, "", dp->s_size, (long long)sp->st_size);
-#endif /* Linux port */
+			(void)printf("%*s%*jd ", 8 - dp->s_size, "",
+			    dp->s_size, (intmax_t)sp->st_size);
 		else
 			printsize(dp->s_size, sp->st_size);
 		if (f_accesstime)
@@ -321,11 +314,7 @@ printcol(DISPLAY *dp)
 		lastentries = dp->entries;
 		if ((array =
 		    realloc(array, dp->entries * sizeof(FTSENT *))) == NULL) {
-#ifndef LINUX /* BSD source */
-			warn(NULL);
-#else /* Linux port */
 			warn("realloc");
-#endif /* Linux port */
 			printscol(dp);
 		}
 	}
@@ -399,12 +388,8 @@ printaname(FTSENT *p, u_long inodefield, u_long sizefield)
 	if (f_inode)
 		chcnt += printf("%*lu ", (int)inodefield, (u_long)sp->st_ino);
 	if (f_size)
-		chcnt += printf("%*lld ",
-#ifndef LINUX /* BSD source */
-		    (int)sizefield, howmany(sp->st_blocks, blocksize));
-#else /* Linux port */
-		    (int)sizefield, (long long)howmany(sp->st_blocks, blocksize));
-#endif /* Linux port */
+		chcnt += printf("%*jd ", (int)sizefield,
+		    (intmax_t)howmany(sp->st_blocks, blocksize));
 #ifdef COLORLS
 	if (f_color)
 		color_printed = colortype(sp->st_mode);
@@ -448,11 +433,11 @@ printtime(time_t ftime)
 		format = d_first ? "%e %b  %Y " : "%b %e  %Y ";
 	strftime(longstring, sizeof(longstring), format, localtime(&ftime));
 	fputs(longstring, stdout);
-/* 
- * Locale-independent implementation for use in ftpd's "internal ls"
- * feature. It's based on the same code from FreeBSD 3.5
- */
-#else
+#else /* (INTERNAL_LS || LINUX) */
+	/* 
+	 * Locale-independent implementation for use in ftpd's "internal ls"
+	 * feature. It's based on the same code from FreeBSD 3.5.
+	 */
 	int i;
 	char longstring[80];
 	static time_t now;
@@ -477,7 +462,7 @@ printtime(time_t ftime)
 			(void)putchar(longstring[i]);
 	}
 	(void)putchar(' ');
-#endif /* (INTERNAL_LS || LINUX) */
+#endif /* (!INTERNAL_LS && !LINUX) */
 }
 
 static int
@@ -701,11 +686,7 @@ printsize(size_t width, off_t bytes)
 			(void)printf("%*.*f%c ", width - 1, dbytes > 10 ? 0 : 1,
 			    dbytes, "BKMGTPE"[unit]);
 	} else
-#ifndef LINUX /* BSD source */
-		(void)printf("%*lld ", width, bytes);
-#else /* Linux port */
-		(void)printf("%*lld ", width, (long long)bytes);
-#endif /* Linux port */
+		(void)printf("%*jd ", width, (intmax_t)bytes);
 }
 
 /*
