@@ -59,6 +59,7 @@ static const char __RCSID_SOURCE[] = "$NetBSD: cmds.c,v 1.30.2.1 1997/11/18 00:5
 #include <err.h>
 #include <glob.h>
 #include <netdb.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -68,6 +69,8 @@ static const char __RCSID_SOURCE[] = "$NetBSD: cmds.c,v 1.30.2.1 1997/11/18 00:5
 #include <signal.h>
 #endif /* LINUX */
 
+#include <port_base.h>
+
 #include "ftp_var.h"
 #include "pathnames.h"
 
@@ -652,16 +655,22 @@ status(argc, argv)
 {
 	int i;
 
-	if (connected)
-		printf("Connected %sto %s.\n",
+	if (connected) {
+		printf("Connected %sto %s",
 		    connected == -1 ? "and logged in" : "", hostname);
-	else
+		if (strcmp(hostaddr, hostname) != 0)
+			printf(" (%s)", hostaddr);
+		printf(".\n");
+	} else
 		puts("Not connected.");
 	if (!proxy) {
 		pswitch(1);
 		if (connected) {
-			printf("Connected for proxy commands to %s.\n",
+			printf("Connected for proxy commands to %s",
 			    hostname);
+			if (strcmp(hostaddr, hostname) != 0)
+				printf(" (%s)", hostaddr);
+			printf(".\n");
 		}
 		else {
 			puts("No proxy connection.");
@@ -673,14 +682,14 @@ status(argc, argv)
 	printf("Passive mode: %s.\n", onoff(passivemode));
 #ifdef USE_SSL
 	if (ssl_secure_flags & SSL_ENABLED) {
-	    printf("TLS/SSL protection of control connection: %s.\n",
+		printf("TLS/SSL protection of control connection: %s.\n",
 		    connected ? onoff(ssl_active_flag) : "no connection");
-	    printf("TLS/SSL protection of data connections: %s.\n",
+		printf("TLS/SSL protection of data connections: %s.\n",
 		    onoff(ssl_encrypt_data));
-	    printf("FTP-SSL compatibility mode: %s.\n",
+		printf("FTP-SSL compatibility mode: %s.\n",
 		    connected ? onoff(ssl_compat_flag) : "(not connected)");
 	}
-#endif /* !USE_SSL */
+#endif /* USE_SSL */
 	printf("Mode: %s; Type: %s; Form: %s; Structure: %s.\n",
 		modename, typename, formname, structname);
 	printf("Verbose: %s; Bell: %s; Prompting: %s; Globbing: %s.\n",
@@ -1283,10 +1292,10 @@ shell(argc, argv)
 			(void)fflush(stdout);
 		}
 		if (argc > 1) {
-			execl(shell, shellnam, "-c", altarg, (char *)0);
+			execl(shell, shellnam, "-c", altarg, NULL);
 		}
 		else {
-			execl(shell, shellnam, (char *)0);
+			execl(shell, shellnam, NULL);
 		}
 		warn("%s", shell);
 		code = -1;
@@ -1605,7 +1614,7 @@ disconnect(argc, argv)
 	connected = 0;
 #ifdef USE_SSL
 	ssl_reinit();
-#endif /*USE_SSL*/
+#endif /* USE_SSL */
 	data = -1;
 	if (!proxy) {
 		macnum = 0;
@@ -2083,8 +2092,8 @@ restart(argc, argv)
 	if (restart_point == 0)
 		puts("No restart point defined");
 	else
-		printf("Restarting at %qd for next get, put or append\n",
-		    (long long)restart_point);
+		printf("Restarting at %jd for next get, put or append\n",
+		    (intmax_t)restart_point);
 }
 
 /* 
@@ -2189,7 +2198,7 @@ sizecmd(argc, argv)
 	}
 	size = remotesize(argv[1], 1);
 	if (size != -1)
-		printf("%s\t%qd\n", argv[1], (long long)size);
+		printf("%s\t%jd\n", argv[1], (intmax_t)size);
 	code = size;
 }
 
@@ -2288,13 +2297,13 @@ feat(argc, argv)
 	int argc;
 	char *argv[];
 {
-	(void)command("FEAT");
+	command("FEAT");
 }
 
 #ifdef USE_SSL
 /*
  * Toggle protection of data connections as defined in RFC2228 and
- * FTP-TLS draft
+ * FTP-TLS draft.
  */
 void
 setprot(argc, argv)
@@ -2309,17 +2318,18 @@ setprot(argc, argv)
 	switch (PBSZ_used_flag) {
 	    case 0:
 		code = command("PBSZ 0");
-		if (code != COMPLETE) break;
-		PBSZ_used_flag=1;
+		if (code != COMPLETE)
+			break;
+		PBSZ_used_flag = 1;
 	    default:
 		code = command(ssl_encrypt_data ? "PROT P" : "PROT C");
 	}
 	if (code == COMPLETE) {
-	    printf("TLS/SSL protection of data connections %s.\n",
+		printf("TLS/SSL protection of data connections %s.\n",
 		    onoff(ssl_encrypt_data));
 	} else {
-	    printf("Protection level of data connections not changed.\n");
-	    ssl_encrypt_data=temp;
+		printf("Protection level of data connections not changed.\n");
+		ssl_encrypt_data = temp;
 	}
 }
 #endif /* USE_SSL */
