@@ -66,15 +66,12 @@ static const char __RCSID_SOURCE[] = "$NetBSD: ftp.c,v 1.29.2.1 1997/11/18 01:01
 #include <err.h>
 #include <errno.h>
 #include <netdb.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#ifdef __STDC__
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 #ifdef LINUX
 #include <signal.h>
@@ -138,6 +135,7 @@ hookup(host0, port)
 	int s, len, tos, error;
 	struct addrinfo hints, *res, *res0;
 	static char hostnamebuf[MAXHOSTNAMELEN];
+	static char hostaddrbuf[NI_MAXHOST];
 	char *host;
 
 	if (*host0 == '[' && strrchr(host0, ']') != NULL) { /*IPv6 addr in []*/
@@ -231,6 +229,22 @@ hookup(host0, port)
 	}
 	memcpy(&hisctladdr, res->ai_addr, res->ai_addrlen);
 	freeaddrinfo(res0);
+
+	if (!getnameinfo((struct sockaddr *)&hisctladdr,
+#ifdef LINUX /* Linux port */
+	    SU_LEN(hisctladdr),
+#else /* BSD source */
+	    hisctladdr.su_len,
+#endif /* BSD source */
+	    hostaddrbuf, sizeof(hostaddrbuf) - 1, NULL, 0, NI_NUMERICHOST))
+		hostaddr = hostaddrbuf;
+	else {
+		hostaddr = NULL;
+		warnx("Abnormal error: connected to host with undeterminable address.");
+		code = -1;
+		goto bad;
+	}
+
 	len = sizeof(myctladdr);
 	if (getsockname(s, (struct sockaddr *)&myctladdr, &len) < 0) {
 		warn("getsockname");
@@ -280,7 +294,7 @@ hookup(host0, port)
 	return (hostname);
 bad:
 	(void)close(s);
-	return ((char *)0);
+	return (NULL);
 }
 
 void
@@ -298,33 +312,21 @@ cmdabort(notused)
 
 /*VARARGS*/
 int
-#ifdef __STDC__
 command(const char *fmt, ...)
-#else
-command(va_alist)
-	va_dcl
-#endif
 {
 	va_list ap;
 	int r;
 	sig_t oldintr;
-#ifndef __STDC__
-	const char *fmt;
-#endif
 
 #ifdef USE_SSL
-        char outputbuf[2048];        /* allow for a 2k command string */
+	/* the size seems to be enough for normal use */
+        char outputbuf[BUFSIZ + MAXPATHLEN + MAXHOSTNAMELEN];
 #endif /* USE_SSL */
 
 	abrtflag = 0;
 	if (debug) {
 		fputs("---> ", stdout);
-#ifdef __STDC__
 		va_start(ap, fmt);
-#else
-		va_start(ap);
-		fmt = va_arg(ap, const char *);
-#endif
 		if (strncmp("PASS ", fmt, 5) == 0)
 			fputs("PASS XXXX", stdout);
 		else if (strncmp("ACCT ", fmt, 5) == 0)
@@ -341,22 +343,17 @@ command(va_alist)
 		return (0);
 	}
 	oldintr = signal(SIGINT, cmdabort);
-#ifdef __STDC__
 	va_start(ap, fmt);
-#else
-	va_start(ap);
-	fmt = va_arg(ap, char *);
-#endif
 
 #ifdef USE_SSL
         /* assemble the output into a buffer */
-	vsprintf(outputbuf,fmt,ap);
-	strcat(outputbuf,"\r\n");
+	vsnprintf(outputbuf, sizeof(outputbuf) - 2, fmt, ap);
+	strcat(outputbuf, "\r\n");
 	if (ssl_active_flag) {
-	    ssl_write(ssl_con,outputbuf,strlen(outputbuf));
+		ssl_write(ssl_con, outputbuf, strlen(outputbuf));
         } else {
-	    fprintf(cout,"%s",outputbuf);
-	    fflush(cout);
+		fprintf(cout, "%s", outputbuf);
+		fflush(cout);
 	}
 #else /* !USE_SSL */
 	vfprintf(cout, fmt, ap);
@@ -407,10 +404,10 @@ getreply(expecteof)
 
 #ifdef USE_SSL
 					if (ssl_active_flag)
-					    ssl_write(ssl_con,buf,3);
+						ssl_write(ssl_con, buf, 3);
 					else
-#endif /* !USE_SSL */
-					    fwrite(buf,3,1,cout);
+#endif /* USE_SSL */
+						fwrite(buf, 3, 1, cout);
 					(void)FFLUSH(cout);
 					break;
 				case DO:
@@ -421,10 +418,10 @@ getreply(expecteof)
 
 #ifdef USE_SSL
 					if (ssl_active_flag)
-					    ssl_write(ssl_con,buf,3);
+						ssl_write(ssl_con, buf, 3);
 					else
-#endif /* !USE_SSL */
-					    fwrite(buf,3,1,cout);
+#endif /* USE_SSL */
+						fwrite(buf, 3, 1, cout);
 					(void)FFLUSH(cout);
 					break;
 				default:
@@ -537,7 +534,7 @@ empty(mask, sec)
 
 	t.tv_sec = (long) sec;
 	t.tv_usec = 0;
-	return (select(32, mask, (fd_set *) 0, (fd_set *) 0, &t));
+	return (select(32, mask, NULL, NULL, &t));
 }
 
 sigjmp_buf	sendabort;
@@ -702,7 +699,7 @@ sendrequest(cmd, local, remote, printnames)
 				(*closefunc)(fin);
 			goto cleanupsend;
 		}
-		if (command("REST %qd", (long long) restart_point) !=
+		if (command("REST %jd", (intmax_t)restart_point) !=
 		    CONTINUE) {
 			if (closefunc != NULL)
 				(*closefunc)(fin);
@@ -749,11 +746,11 @@ sendrequest(cmd, local, remote, printnames)
 			bytes += c;
 #ifdef USE_SSL
                         if (ssl_data_active_flag) {
-			    for (bufp = buf; c > 0; c -= d, bufp += d)
-				if ((d = ssl_write(ssl_data_con, bufp, c)) <= 0)
-					break;
+				for (bufp = buf; c > 0; c -= d, bufp += d)
+					if ((d = ssl_write(ssl_data_con, bufp, c)) <= 0)
+						break;
 			} else 
-#endif /* !USE_SSL */
+#endif /* USE_SSL */
 			{
 			for (bufp = buf; c > 0; c -= d, bufp += d)
 				if ((d = write(fileno(dout), bufp, c)) <= 0)
@@ -830,10 +827,10 @@ sendrequest(cmd, local, remote, printnames)
 	(void)fclose(dout);
 
 #ifdef USE_SSL
-	if (ssl_data_active_flag && (ssl_data_con!=NULL)) {
-	    SSL_free(ssl_data_con);
-	    ssl_data_active_flag=0;
-	    ssl_data_con=NULL;
+	if (ssl_data_active_flag && (ssl_data_con != NULL)) {
+		SSL_free(ssl_data_con);
+		ssl_data_active_flag = 0;
+		ssl_data_con = NULL;
 	}
 #endif /* USE_SSL */
 
@@ -866,10 +863,10 @@ abort:
 		(void)fclose(dout);
 
 #ifdef USE_SSL
-	if (ssl_data_active_flag && (ssl_data_con!=NULL)) {
-	    SSL_free(ssl_data_con);
-	    ssl_data_active_flag=0;
-	    ssl_data_con=NULL;
+	if (ssl_data_active_flag && (ssl_data_con != NULL)) {
+		SSL_free(ssl_data_con);
+		ssl_data_active_flag = 0;
+		ssl_data_con = NULL;
 	}
 #endif /* USE_SSL */
 
@@ -1051,7 +1048,7 @@ recvrequest(cmd, local, remote, lmode, printnames, ignorespecial)
 	if (sigsetjmp(recvabort,1))
 		goto abort;
 	if (is_retr && restart_point &&
-	    command("REST %qd", (long long) restart_point) != CONTINUE)
+	    command("REST %jd", (intmax_t)restart_point) != CONTINUE)
 		return;
 	if (remote) {
 		if (command("%s %s", cmd, remote) != PRELIM) {
@@ -1129,25 +1126,25 @@ recvrequest(cmd, local, remote, lmode, printnames, ignorespecial)
 		errno = d = 0;
 #ifdef USE_SSL
                 if (ssl_data_active_flag) {
-		    while ((c = ssl_read(ssl_data_con, buf, bufsize)) > 0) {
-			    if ((d = write(fileno(fout), buf, c)) != c)
-				    break;
-			    bytes += c;
-			    if (hash) {
-				    while (bytes >= hashbytes) {
-					    (void) putchar('#');
-					    hashbytes += HASHBYTES;
-				    }
-				    (void) fflush(stdout);
-			    }
-		    }
-		    if ( c < -1 ) {
-			/* tell the user ... who else */
-			ssl_log_err(bio_err, "ssl_read DATA error");
-			warnx("TLS/SSL DATA connection error");
-		    }
+			while ((c = ssl_read(ssl_data_con, buf, bufsize)) > 0) {
+				if ((d = write(fileno(fout), buf, c)) != c)
+					break;
+				bytes += c;
+				if (hash) {
+					while (bytes >= hashbytes) {
+						putchar('#');
+						hashbytes += HASHBYTES;
+					}
+					fflush(stdout);
+				}
+			}
+			if (c < -1) {
+				/* tell the user ... who else */
+				ssl_log_err(bio_err, "ssl_read DATA error");
+				warnx("TLS/SSL DATA connection error");
+			}
 		} else 
-#endif /* !USE_SSL */
+#endif /* USE_SSL */
 		{
 		while ((c = read(fileno(din), buf, bufsize)) > 0) {
 			if ((d = write(fileno(fout), buf, c)) != c)
@@ -1267,10 +1264,10 @@ break2:
 	(void)fclose(din);
 
 #ifdef USE_SSL
-	if (ssl_data_active_flag && (ssl_data_con!=NULL)) {
-	    SSL_free(ssl_data_con);
-	    ssl_data_active_flag=0;
-	    ssl_data_con=NULL;
+	if (ssl_data_active_flag && (ssl_data_con != NULL)) {
+		SSL_free(ssl_data_con);
+		ssl_data_active_flag = 0;
+		ssl_data_con = NULL;
 	}
 #endif /* USE_SSL */
 
@@ -1281,8 +1278,7 @@ break2:
 		if (preserve && (closefunc == fclose)) {
 			mtime = remotemodtime(remote, 0);
 			if (mtime != -1) {
-				(void)gettimeofday(&tval[0],
-				    (struct timezone *)0);
+				(void)gettimeofday(&tval[0], NULL);
 				tval[1].tv_sec = mtime;
 				tval[1].tv_usec = 0;
 				if (utimes(local, tval) == -1) {
@@ -1325,10 +1321,10 @@ abort:
 		(void)fclose(din);
 
 #ifdef USE_SSL
-	if (ssl_data_active_flag && (ssl_data_con!=NULL)) {
-	    SSL_free(ssl_data_con);
-	    ssl_data_active_flag=0;
-	    ssl_data_con=NULL;
+	if (ssl_data_active_flag && (ssl_data_con != NULL)) {
+		SSL_free(ssl_data_con);
+		ssl_data_active_flag = 0;
+		ssl_data_con = NULL;
 	}
 	/* clear queued command replies if present */
 	reset(0, NULL);
@@ -1544,6 +1540,7 @@ initconn()
 			prt[1] &= 0xff;
 			error = 0;
 epsv_done:
+			;
 		} else
 			error = 1;
 
@@ -1556,7 +1553,7 @@ epsv_done:
 		data_addr.su_port = htons(pack2(prt, 0));
 		if (connect(data, (struct sockaddr *)&data_addr,
 #ifdef LINUX
-			    sizeof(struct sockaddr)) < 0) {
+			    SU_LEN(data_addr)) < 0) {
 #else /* BSD source */
 			    data_addr.su_len) < 0) {
 #endif /* BSD source */
@@ -1618,7 +1615,7 @@ noport:
 
 	if (bind(data, (struct sockaddr *)&data_addr,
 #ifdef LINUX
-		 sizeof(struct sockaddr)) < 0) {
+		 SU_LEN(data_addr)) < 0) {
 #else /* BSD source */
 		 data_addr.su_len) < 0) {
 #endif /* BSD source */
@@ -1646,7 +1643,9 @@ noport:
 		if (data_addr.su_family == AF_INET6 &&
 		    IN6_IS_ADDR_V4MAPPED(&data_addr.su_sin6.sin6_addr)) {
 			memset(&data_addr4, 0, sizeof(data_addr4));
+#ifndef LINUX /* BSD source */
 			data_addr4.sin_len = sizeof(struct sockaddr_in);
+#endif /* BSD source */
 			data_addr4.sin_family = AF_INET;
 			data_addr4.sin_port = data_addr.su_port;
 			memcpy((caddr_t)&data_addr4.sin_addr,
@@ -1667,7 +1666,7 @@ noport:
 				af = (daddr->su_family == AF_INET) ? 1 : 2;
 				if (getnameinfo((struct sockaddr *)daddr,
 #ifdef LINUX /* Linux port */
-						sizeof(struct sockaddr),
+						SU_LEN(*daddr),
 #else /* BSD source */
 						daddr->su_len,
 #endif /* BSD source */
@@ -1692,7 +1691,12 @@ noport:
 				if (daddr->su_family == AF_INET6)
 					daddr->su_sin6.sin6_scope_id = 0;
 				if (getnameinfo((struct sockaddr *)daddr,
-						daddr->su_len, hname,
+#ifdef LINUX /* Linux port */
+						SU_LEN(*daddr),
+#else /* BSD source */
+						daddr->su_len,
+#endif /* BSD source */
+						hname,
 						sizeof(hname) - 1, NULL, 0,
 						NI_NUMERICHOST)) {
 					result = ERROR;
@@ -1780,107 +1784,108 @@ dataconn(lmode)
 #endif /* USE_SSL */
 
 #ifdef LINUX /* Linux port */
-	fromlen = sizeof(struct sockaddr);
+	fromlen = SU_LEN(myctladdr);
 #else /* BSD source */
 	fromlen = myctladdr.su_len;
 #endif /* BSD source */
 
 	if (passivemode) {
 #ifdef USE_SSL
-    		ssl_data_active_flag=0;
+		ssl_data_active_flag = 0;
 		if (ssl_active_flag && ssl_encrypt_data) {
-		    /* do SSL */
-		    if (ssl_data_con!=NULL) {
-	    		SSL_free(ssl_data_con);
-	    		ssl_data_con=NULL;
-		    }
-		    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
-		    SSL_set_connect_state(ssl_data_con);
+			/* Do SSL */
+			if (ssl_data_con != NULL) {
+	    			SSL_free(ssl_data_con);
+	    			ssl_data_con = NULL;
+			}
+			ssl_data_con = (SSL *)SSL_new(ssl_ctx);
+			SSL_set_connect_state(ssl_data_con);
 
-		    SSL_set_fd(ssl_data_con,data);
-		    set_ssl_trace(ssl_data_con);
+			SSL_set_fd(ssl_data_con, data);
+			set_ssl_trace(ssl_data_con);
 
-		    /* this is the "magic" call that makes 
-		     * this quick assuming Eric has this going
-	             * okay! ;-)
-	             */
-		    SSL_copy_session_id(ssl_data_con,ssl_con);
+			/*
+			 * This is the "magic" call that makes 
+			 * this quick assuming Eric has this going
+	        	 * okay! ;-)
+	        	 */
+			SSL_copy_session_id(ssl_data_con, ssl_con);
 
-		    /* we are doing I/O and not using select so 
-	             * it is "safe" to read ahead
-	             */
-		    /* SSL_set_read_ahead(ssl_data_con,1);*/ /**/
+			/*
+			 * We are doing I/O and not using select so 
+	    	         * it is "safe" to read ahead.
+	        	 */
+			/* SSL_set_read_ahead(ssl_data_con,1);*/ /**/
 
-		    if (debug) {
-			fprintf(stderr,"===>START SSL connect on DATA\n");
-			fflush(stderr);
-		    }
+			if (debug)
+				warnx("===> START SSL connect on DATA");
 
-		    if ((ret=SSL_connect(ssl_data_con))<=0) {
-			/* tell the user ... who else */
-	    		ssl_log_err(bio_err,"SSL_connect DATA error %d ",ret);
-	    		warnx("TLS/SSL DATA connection failed");
+			if ((ret=SSL_connect(ssl_data_con)) <= 0) {
+				/* tell the user ... who else */
+	    			ssl_log_err(bio_err,
+					    "SSL_connect DATA error %d", ret);
+	    			warnx("TLS/SSL DATA connection failed");
 
-			/* abort time methinks ... */
-			close(data);
-			return NULL;
-		    } else {
-			if (ssl_debug_flag) {
-			    ssl_version=SSL_get_cipher_version(ssl_data_con);
-			    ssl_cipher=SSL_get_current_cipher(ssl_data_con);
-			    SSL_CIPHER_get_bits(ssl_cipher, &ssl_bits);
-			    ssl_log_msgn(bio_err,
-					"[DATA: %s, cipher %s, %d bits]",
-					ssl_version, 
-					SSL_CIPHER_get_name(ssl_cipher),
-					ssl_bits);
+				/* abort time methinks ... */
+				close(data);
+				return NULL;
+			} else {
+				if (ssl_debug_flag) {
+					ssl_version = SSL_get_cipher_version(ssl_data_con);
+					ssl_cipher = SSL_get_current_cipher(ssl_data_con);
+					SSL_CIPHER_get_bits(ssl_cipher, &ssl_bits);
+					ssl_log_msgn(bio_err,
+					    "[DATA: %s, cipher %s, %d bits]",
+					    ssl_version, 
+					    SSL_CIPHER_get_name(ssl_cipher),
+					    ssl_bits);
+				}
+
+				/*
+				 * Get server certificates of control and data
+				 * connections.
+				 */
+				x509_ssl_con = SSL_get_peer_certificate(ssl_con);
+				x509_ssl_data_con = SSL_get_peer_certificate(ssl_data_con);
+
+				/*
+				 * Check the certificates if server certificate
+				 * is present for control connection.
+				 */
+				switch (ssl_X509_cmp(x509_ssl_con,
+				    x509_ssl_data_con)) {
+				case -3:
+					warnx("server did not present a certificate for data connection");
+
+					/* Drop an established TLS/SSL connection. */
+					SSL_free(ssl_data_con);
+					ssl_data_con = NULL;
+					close(data);
+					data = -1;
+
+					return NULL;
+				case 0:
+				        warnx("server certificates for control and data connections are different");
+
+					/* Drop an established TLS/SSL connection. */
+					SSL_free(ssl_data_con);
+					ssl_data_con = NULL;
+					close(data);
+					data = -1;
+
+					return NULL;
+				default:
+					break;
+				}
+
+				X509_free(x509_ssl_con);
+				X509_free(x509_ssl_data_con);
+
+				ssl_data_active_flag = 1;
 			}
 
-			/* get server certificates of control and data
-			 * connections 
-			 */
-			x509_ssl_con=SSL_get_peer_certificate(ssl_con);
-			x509_ssl_data_con=SSL_get_peer_certificate(ssl_data_con);
-
-			/* check the certificates if server certificate
-			 * is present for control connection 
-			 */
-			switch (ssl_X509_cmp(x509_ssl_con, x509_ssl_data_con)) {
-			case -3:
-			    warnx("server did not present a certificate for data connection");
-
-			    /* drop established TLS/SSL connection */
-			    SSL_free(ssl_data_con);
-			    ssl_data_con=NULL;
-			    (void) close(data);
-			    data=-1;
-
-			    return NULL;
-			case  0:
-			    warnx("server certificates for control and data connections are different");
-
-			    /* drop established TLS/SSL connection */
-			    SSL_free(ssl_data_con);
-			    ssl_data_con=NULL;
-			    (void) close(data);
-			    data=-1;
-
-			    return NULL;
-			default:
-			}
-
-			X509_free(x509_ssl_con);
-			X509_free(x509_ssl_data_con);
-
-			ssl_data_active_flag=1;
-		    }
-
-		    if (debug) {
-			fprintf(stderr,"===>DONE SSL connect on DATA %d\n",
-				data);
-			fflush(stderr);
-		    }
-
+			if (debug)
+				warnx("===> DONE SSL connect on DATA %d", data);
 		}
 #endif /* USE_SSL */
 
@@ -1904,97 +1909,98 @@ dataconn(lmode)
 #endif
 
 #ifdef USE_SSL
-	ssl_data_active_flag=0;
+	ssl_data_active_flag = 0;
 	if (ssl_active_flag && ssl_encrypt_data) {
-	    /* do SSL */
-	    if (ssl_data_con!=NULL) {
-	      SSL_free(ssl_data_con);
-	      ssl_data_con=NULL;
-	    }
-	    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
-	    SSL_set_connect_state(ssl_data_con);
+		/* Do SSL */
+		if (ssl_data_con != NULL) {
+			SSL_free(ssl_data_con);
+			ssl_data_con = NULL;
+		}
+		ssl_data_con = (SSL *)SSL_new(ssl_ctx);
+		SSL_set_connect_state(ssl_data_con);
 
-	    SSL_set_fd(ssl_data_con,data);
-	    set_ssl_trace(ssl_data_con);
+		SSL_set_fd(ssl_data_con, data);
+		set_ssl_trace(ssl_data_con);
 
-	    /* this is the "magic" call that makes 
-	     * this quick assuming Eric has this going
-	     * okay! ;-)
-	     */
-	    SSL_copy_session_id(ssl_data_con,ssl_con);
+		/*
+		 * This is the "magic" call that makes 
+		 * this quick assuming Eric has this going
+		 * okay! ;-)
+		 */
+		SSL_copy_session_id(ssl_data_con, ssl_con);
 
-	    /* we are doing I/O and not using select so 
-	     * it is "safe" to read ahead
-	     */
-	    /* SSL_set_read_ahead(ssl_data_con,1);*/ /**/
+		/*
+		 * We are doing I/O and not using select so 
+		 * it is "safe" to read ahead.
+		 */
+		/* SSL_set_read_ahead(ssl_data_con,1);*/
 
-	    if (debug) {
-		fprintf(stderr,"===>START SSL connect on DATA\n");
-		fflush(stderr);
-	    }
+		if (debug)
+			warnx("===> START SSL connect on DATA");
 
-	    if ((ret=SSL_connect(ssl_data_con))<=0) {
-		/* tell the user ... who else */
-	    	ssl_log_err(bio_err,"SSL_connect DATA error %d ",ret);
-	    	warnx("TLS/SSL DATA connection failed");
+		if ((ret = SSL_connect(ssl_data_con)) <= 0) {
+			/* tell the user ... who else */
+	    		ssl_log_err(bio_err, "SSL_connect DATA error %d", ret);
+	    		warnx("TLS/SSL DATA connection failed");
 
-		/* abort time methinks ... */
-		close(data);
-		return NULL;
-	    } else {
-		if (ssl_debug_flag) {
-		    ssl_version=SSL_get_cipher_version(ssl_data_con);
-		    ssl_cipher=SSL_get_current_cipher(ssl_data_con);
-		    SSL_CIPHER_get_bits(ssl_cipher, &ssl_bits);
-		    ssl_log_msgn(bio_err, "[DATA: %s, cipher %s, %d bits]",
-				ssl_version, SSL_CIPHER_get_name(ssl_cipher),
-				ssl_bits);
+			/* abort time methinks ... */
+			close(data);
+			return NULL;
+		} else {
+			if (ssl_debug_flag) {
+				ssl_version = SSL_get_cipher_version(ssl_data_con);
+				ssl_cipher = SSL_get_current_cipher(ssl_data_con);
+				SSL_CIPHER_get_bits(ssl_cipher, &ssl_bits);
+				ssl_log_msgn(bio_err,
+				    "[DATA: %s, cipher %s, %d bits]",
+				    ssl_version,
+				    SSL_CIPHER_get_name(ssl_cipher), ssl_bits);
+			}
+
+			/*
+			 * Get server certificates of control and data
+			 * connections.
+			 */
+			x509_ssl_con = SSL_get_peer_certificate(ssl_con);
+			x509_ssl_data_con = SSL_get_peer_certificate(ssl_data_con);
+
+			/*
+			 * Check the certificates if server certificate
+			 * is present for control connection.
+			 */
+			switch (ssl_X509_cmp(x509_ssl_con, x509_ssl_data_con)) {
+			case -3:
+				warnx("server did not present a certificate for data connection");
+
+				/* Drop an established TLS/SSL connection. */
+				SSL_free(ssl_data_con);
+				ssl_data_con = NULL;
+				close(data);
+				data = -1;
+
+				return NULL;
+			case 0:
+				warnx("server certificates for control and data connections are different");
+
+				/* Drop an established TLS/SSL connection. */
+				SSL_free(ssl_data_con);
+				ssl_data_con = NULL;
+				close(data);
+				data = -1;
+
+				return NULL;
+			default:
+				break;
+			}
+
+			X509_free(x509_ssl_con);
+			X509_free(x509_ssl_data_con);
+
+			ssl_data_active_flag = 1;
 		}
 
-		/* get server certificates of control and data
-		 * connections 
-		 */
-		x509_ssl_con=SSL_get_peer_certificate(ssl_con);
-		x509_ssl_data_con=SSL_get_peer_certificate(ssl_data_con);
-
-		/* check the certificates if server certificate
-		 * is present for control connection 
-		 */
-		switch (ssl_X509_cmp(x509_ssl_con, x509_ssl_data_con)) {
-		case -3:
-		    warnx("server did not present a certificate for data connection");
-
-		    /* drop established TLS/SSL connection */
-		    SSL_free(ssl_data_con);
-		    ssl_data_con=NULL;
-		    (void) close(data);
-		    data=-1;
-
-		    return NULL;
-		case  0:
-		    warnx("server certificates for control and data connections are different");
-
-		    /* drop established TLS/SSL connection */
-		    SSL_free(ssl_data_con);
-		    ssl_data_con=NULL;
-		    (void) close(data);
-		    data=-1;
-
-		    return NULL;
-		default:
-		}
-
-		X509_free(x509_ssl_con);
-		X509_free(x509_ssl_data_con);
-
-		ssl_data_active_flag=1;
-	    }
-
-	    if (debug) {
-		fprintf(stderr,"===>DONE SSL connect on DATA %d\n",data);
-		fflush(stderr);
-	    }
-
+		if (debug)
+			warnx("===> DONE SSL connect on DATA %d", data);
 	}
 #endif /* USE_SSL */
 
@@ -2027,6 +2033,7 @@ pswitch(flag)
 	static struct comvars {
 		int connect;
 		char name[MAXHOSTNAMELEN];
+		char addr[NI_MAXHOST];
 		union sockunion mctl;
 		union sockunion hctl;
 		FILE *in;
@@ -2076,6 +2083,12 @@ pswitch(flag)
 	} else
 		ip->name[0] = '\0';
 	hostname = op->name;
+	if (hostaddr) {
+		(void)strncpy(ip->addr, hostaddr, sizeof(ip->addr) - 1);
+		ip->addr[sizeof(ip->addr) - 1] = '\0';
+	} else
+		ip->addr[0] = '\0';
+	hostaddr = op->addr;
 	ip->hctl = hisctladdr;
 	hisctladdr = op->hctl;
 	ip->mctl = myctladdr;
@@ -2312,7 +2325,7 @@ gunique(local)
 		*cp = '/';
 	if (d < 0) {
 		warn("local: %s", local);
-		return ((char *) 0);
+		return (NULL);
 	}
 	(void)strcpy(new, local);
 	cp = new + strlen(new);
@@ -2320,7 +2333,7 @@ gunique(local)
 	while (!d) {
 		if (++count == 100) {
 			puts("runique: can't find unique file name.");
-			return ((char *) 0);
+			return (NULL);
 		}
 		*cp++ = ext;
 		*cp = '\0';
@@ -2367,22 +2380,19 @@ abort_remote(din)
 #ifdef USE_SSL
 	if (!ssl_active_flag) {
 #endif /*USE_SSL*/
-	sprintf(buf, "%c%c%c", IAC, IP, IAC);
+	snprintf(buf, sizeof(buf), "%c%c%c", IAC, IP, IAC);
 	if (send(fileno(cout), buf, 3, MSG_OOB) != 3)
 		warn("abort");
-#ifdef USE_SSL
-	}
-	if (ssl_active_flag) {
-		sprintf(buf, "%c%c%c", IAC, IP, IAC);
-		ssl_write(ssl_con, buf, strlen(buf));
-		sprintf(buf, "%cABOR\r\n", DM);
-		ssl_write(ssl_con, buf, strlen(buf));
-	} else
-#endif /*USE_SSL*/
-	{
 	fprintf(cout, "%cABOR\r\n", DM);
 	(void)fflush(cout);
+#ifdef USE_SSL
+	} else {
+		snprintf(buf, sizeof(buf), "%c%c%c", IAC, IP, IAC);
+		ssl_write(ssl_con, buf, strlen(buf));
+		snprintf(buf, sizeof(buf), "%cABOR\r\n", DM);
+		ssl_write(ssl_con, buf, strlen(buf));
 	}
+#endif /*USE_SSL*/
 
 	FD_ZERO(&mask);
 
@@ -2398,20 +2408,22 @@ abort_remote(din)
 		if (ptabflg)
 			code = -1;
 #ifdef USE_SSL
-		/* Check the possible exception: the abort command is received
+		/*
+		 * Check the possible exception: the abort command is received
 		 * during an initialization of the TLS/SSL session on the data
-		 * connection */
+		 * connection.
+		 */
 		if (ssl_active_flag && ssl_encrypt_data &&
-		    !ssl_data_active_flag && (ssl_data_con!=NULL)) {
-			/* drop SSL connection */
+		    !ssl_data_active_flag && (ssl_data_con != NULL)) {
+			/* Drop the SSL connection. */
 			SSL_free(ssl_data_con);
-			ssl_data_active_flag=0;
-			ssl_data_con=NULL;
+			ssl_data_active_flag = 0;
+			ssl_data_con = NULL;
 
 			/* close socket */
 			if (data >= 0) {
-				(void)shutdown(data, 1+1);
-				(void)close(data);
+				shutdown(data, 1 + 1);
+				close(data);
 				data = -1;
 			}
 		} else
