@@ -76,15 +76,16 @@ static const char __RCSID_SOURCE[] = "$NetBSD: main.c,v 1.26 1997/10/14 16:31:22
 
 #ifdef LINUX
 #include <signal.h>
-#endif
+#endif /* LINUX */
+
+#include <port_base.h>
 
 #include "ftp_var.h"
 #include "pathnames.h"
 
-#ifdef USE_SSL
-
+#ifdef USE_SSL /*SKYNICK XXX*/
 long
-bio_dump_cb(bio,cmd,argp,argi,argl,ret)
+bio_dump_cb(bio, cmd, argp, argi, argl, ret)
 	BIO *bio;
 	int cmd;
 	char *argp;
@@ -92,43 +93,39 @@ bio_dump_cb(bio,cmd,argp,argi,argl,ret)
 	long argl;
 	long ret;
 {
-        BIO *out;
+	BIO *out;
 
 /*
-        out=(BIO *)BIO_get_callback_arg(bio);
+	out=(BIO *)BIO_get_callback_arg(bio);
 */
-	out=bio_err;
-        if (out == NULL) return(ret);
+	out = bio_err;
+	if (out == NULL) return(ret);
 
-        if (cmd == (BIO_CB_READ|BIO_CB_RETURN))
-                {
-                BIO_printf(out,"read from %08X (%d bytes => %ld (%X))\n",
-                        bio,argi,ret,ret);
-                BIO_dump(out,argp,(int)ret);
+	if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
+		BIO_printf(out, "read from %08X (%d bytes => %ld (%X))\n",
+			  bio, argi, ret, ret);
+		BIO_dump(out,argp,(int)ret);
 		BIO_flush(out);
-                }
-        else if (cmd == (BIO_CB_WRITE|BIO_CB_RETURN))
-                {
-                BIO_printf(out,"write to %08X (%d bytes => %ld (%X))\n",
-                        bio,argi,ret,ret);
-                BIO_dump(out,argp,(int)ret);
+	} else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
+		BIO_printf(out, "write to %08X (%d bytes => %ld (%X))\n",
+			  bio, argi, ret, ret);
+		BIO_dump(out, argp, (int)ret);
 		BIO_flush(out);
-                }
-        return( (cmd & BIO_CB_RETURN) ? ret : 1);
+	}
+	return((cmd & BIO_CB_RETURN) ? ret : 1);
 }
 
 int
 set_ssl_trace(SSL *con) 
 {
-    if (con!=NULL) {
-	if (ssl_debug_flag) {
-	    BIO_set_callback(SSL_get_rbio(con),bio_dump_cb);
-	    BIO_set_callback_arg(SSL_get_rbio(con),bio_err);
+	if (con != NULL) {
+		if (ssl_debug_flag) {
+			BIO_set_callback(SSL_get_rbio(con), bio_dump_cb);
+			BIO_set_callback_arg(SSL_get_rbio(con), bio_err);
+		}
 	}
-    }
-    return 0;
+	return 0;
 }
-
 #endif /* USE_SSL */
 
 #ifdef INET6
@@ -222,6 +219,8 @@ main(argc, argv)
 
 	while ((ch = getopt(argc, argv, "46adeginpP:s:tUvVz:")) != -1) {
 		switch (ch) {
+			char *optname;
+			size_t optnamelen;
 		case '4':
 			family = AF_INET;
 			break;
@@ -288,70 +287,88 @@ main(argc, argv)
 
 #ifdef USE_SSL
 		case 'z':
-			if ( (strcmp(optarg, "tls") == 0 ) || 
-			     (strcmp(optarg, "nossl") == 0 ) ) {
-			    SSL_secure_flags_ON(SSL_USE_TLS);
-			    SSL_secure_flags_OFF(SSL_USE_COMPAT);
+			if ((strcmp(optarg, "tls") == 0) || 
+			    (strcmp(optarg, "nossl") == 0)) {
+				SSL_secure_flags_ON(SSL_USE_TLS);
+				SSL_secure_flags_OFF(SSL_USE_COMPAT);
 			}
-			if ( (strcmp(optarg, "ssl") == 0 ) || 
-			     (strcmp(optarg, "notls") == 0 ) ) {
-			    SSL_secure_flags_ON(SSL_USE_COMPAT);
-			    SSL_secure_flags_OFF(SSL_USE_TLS);
+			if ((strcmp(optarg, "ssl") == 0) || 
+			    (strcmp(optarg, "notls") == 0)) {
+				SSL_secure_flags_ON(SSL_USE_COMPAT);
+				SSL_secure_flags_OFF(SSL_USE_TLS);
 			}
-			if (strcmp(optarg, "secure") == 0 ) {
-			    SSL_secure_flags_ON(SSL_ENABLED);
-			    SSL_secure_flags_OFF(SSL_USE_NONSECURE);
+			if (strcmp(optarg, "secure") == 0) {
+				SSL_secure_flags_ON(SSL_ENABLED);
+				SSL_secure_flags_OFF(SSL_USE_NONSECURE);
 			}
 			/* disable *all* ssl stuff */
-			if (strcmp(optarg, "nosecure") == 0 ) {
-			    SSL_secure_flags_ON(SSL_USE_NONSECURE);
-			    SSL_secure_flags_OFF(SSL_ENABLED);
+			if (strcmp(optarg, "nosecure") == 0) {
+				SSL_secure_flags_ON(SSL_USE_NONSECURE);
+				SSL_secure_flags_OFF(SSL_ENABLED);
 			}
-			if (strncmp(optarg, "verify=", strlen("verify=")) == 0 ) {
-		    	    switch ( atoi(optarg+strlen("verify=")) ) {
-			    case 0:
-				ssl_verify_flag = SSL_VERIFY_NONE;
-				break;
-			    case 1:
-				ssl_verify_flag = SSL_VERIFY_PEER;
-				break;
-			    default:
-				warnx("unknown verify level \"%d\" ignored",
-				    atoi(optarg+strlen("verify=")));
-			    }
+			optname = "verify=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+		    		switch (atoi(optarg + optnamelen)) {
+				case 0:
+					ssl_verify_flag = SSL_VERIFY_NONE;
+					break;
+				case 1:
+					ssl_verify_flag = SSL_VERIFY_PEER;
+					break;
+				default:
+					warnx("unknown verify level ignored: %d",
+					atoi(optarg + optnamelen));
+				}
 			}
-			if (strncmp(optarg, "cert=", strlen("cert=")) == 0 ) {
-			    ssl_cert_file=optarg+strlen("cert=");
+			optname = "cert=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_cert_file = optarg + optnamelen;
 			}
-			if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
-			    ssl_key_file=optarg+strlen("key=");
+			optname = "key=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_key_file = optarg + optnamelen;
 			}
-			if (strncmp(optarg, "logfile=", strlen("logfile=")) == 0 ) {
-			    ssl_log_file=optarg+strlen("logfile=");
+			optname = "logfile=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_log_file = optarg + optnamelen;
 			}
 			if (strcmp(optarg, "debug") == 0 ) {
-			    ssl_debug_flag=1;
+				ssl_debug_flag = 1;
 			}
-			if (strncmp(optarg, "cipher=", strlen("cipher=")) == 0 ) {
-			    ssl_cipher_list=optarg+strlen("cipher=");
+			optname = "cipher=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_cipher_list = optarg + optnamelen;
 			}
-			if (strncmp(optarg, "CAfile=", strlen("CAfile=")) == 0 ) {
-			    ssl_CA_file=optarg+strlen("CAfile=");
+			optname = "CAfile=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_CA_file = optarg + optnamelen;
 			}
-			if (strncmp(optarg, "CApath=", strlen("CApath=")) == 0 ) {
-			    ssl_CA_path=optarg+strlen("CApath=");
+			optname = "CApath=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_CA_path = optarg + optnamelen;
 			}
-			if (strncmp(optarg, "CRLfile=", strlen("CRLfile=")) == 0 ) {
-			    ssl_CRL_file=optarg+strlen("CRLfile=");
+			optname = "CRLfile=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_CRL_file = optarg + optnamelen;
 			}
-			if (strncmp(optarg, "CRLpath=", strlen("CRLpath=")) == 0 ) {
-			    ssl_CRL_path=optarg+strlen("CRLpath=");
+			optname = "CRLpath=";
+			optnamelen = strlen(optname);
+			if (strncmp(optarg, optname, optnamelen) == 0) {
+				ssl_CRL_path = optarg + optnamelen;
 			}
-			if (strcmp(optarg, "noprot") == 0 ) {
-			    ssl_tryprot_flag=0;
+			if (strcmp(optarg, "noprot") == 0) {
+				ssl_tryprot_flag = 0;
 			}
-			if (strcmp(optarg, "verbose") == 0 ) {
-			    ssl_verbose_flag=1;
+			if (strcmp(optarg, "verbose") == 0) {
+				ssl_verbose_flag = 1;
 			}
 			break;
 #endif /* USE_SSL */
@@ -405,19 +422,23 @@ main(argc, argv)
 
 #ifdef USE_SSL
         if (ssl_secure_flags & SSL_ENABLED) {
-	    if (!do_ssleay_init(0)) {
-		warnx("TLS/SSL initialization failed");
-		if (ssl_debug_flag) {
-		    /* Print information about all recorded errors
-		     */
-		    ssl_log_msgn(bio_err, "displaying the rest of SSL errors");
-		    while (ERR_peek_error()) {
-			ssl_log_warn_debug(bio_err, "error queue");
-		    }
-		    ssl_log_msgn(bio_err, "error queue flushed, exit");
+		if (!do_ssleay_init(0)) {
+			warnx("TLS/SSL initialization failed");
+			if (ssl_debug_flag) {
+				/*
+				 * Print information about all recorded errors.
+				 */
+				ssl_log_msgn(bio_err,
+					"displaying the rest of SSL errors");
+				while (ERR_peek_error()) {
+					ssl_log_warn_debug(bio_err,
+							  "error queue");
+				}
+				ssl_log_msgn(bio_err,
+					"error queue flushed, exit");
+			}
+			exit(1);
 		}
-		exit(1);
-	    }
 	}
 #endif /* USE_SSL */
 
@@ -807,12 +828,12 @@ OUT:
 			break;
 		case 1:
 			slrflag++;
-			altarg = (char *) 0;
+			altarg = NULL;
 			break;
 		default:
 			break;
 	}
-	return ((char *)0);
+	return (NULL);
 }
 
 /*
@@ -849,7 +870,7 @@ help(argc, argv)
 		c = getcmd(arg);
 		if (c == (struct cmd *)-1)
 			printf("?Ambiguous help command %s\n", arg);
-		else if (c == (struct cmd *)0)
+		else if (c == NULL)
 			printf("?Invalid help command %s\n", arg);
 		else
 			printf("%-*s\t%s\n", HELPINDENT,
