@@ -55,6 +55,7 @@ static const char __RCSID_SOURCE[] = "$NetBSD: util.c,v 1.16.2.1 1997/11/18 01:0
 #include <glob.h>
 #include <limits.h>
 #include <pwd.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -66,9 +67,10 @@ static const char __RCSID_SOURCE[] = "$NetBSD: util.c,v 1.16.2.1 1997/11/18 01:0
 
 #ifdef LINUX
 #include <signal.h>
-#include <linuxport.h>
 #endif /* LINUX */
 
+#include <port_base.h>
+
 #include "ftp_var.h"
 #include "pathnames.h"
 
@@ -272,14 +274,14 @@ login(host, user, pass)
 	}
 #ifdef USE_SSL
 	if (ssl_secure_flags & SSL_ENABLED) {
-	    n = auth();
-	    if (/* TLS/SSL handshake failed */
-		(n<0)
-	        /* TLS/SSL session negotiation failed */
-		|| (!(ssl_secure_flags & SSL_USE_NONSECURE) && !n) )
-		return 0;
+		n = auth();
+		if (/* TLS/SSL handshake failed */
+		    (n < 0)
+	            /* TLS/SSL session negotiation failed */
+		    || (!(ssl_secure_flags & SSL_USE_NONSECURE) && !n))
+			return 0;
 	}
-#endif /*USE_SSL*/
+#endif /* USE_SSL */
 	n = command("USER %s", user);
 	if (n == CONTINUE) {
 		if (pass == NULL)
@@ -641,17 +643,17 @@ progressmeter(flag)
 	len = 0;
 
 	if (flag == -1) {
-		(void)gettimeofday(&start, (struct timezone *)0);
+		(void)gettimeofday(&start, NULL);
 		lastupdate = start;
 		lastsize = restart_point;
 	}
-	(void)gettimeofday(&now, (struct timezone *)0);
+	(void)gettimeofday(&now, NULL);
 	if (!progress || filesize <= 0)
 		return;
 	cursize = bytes + restart_point;
 
 #ifdef LINUX /* Linux port */
-	ratio = (long long)cursize * 100 / filesize;
+	ratio = (intmax_t)cursize * 100 / filesize;
 #else /* BSD source */
 	ratio = cursize * 100 / filesize;
 #endif /* BSD source */
@@ -676,7 +678,7 @@ progressmeter(flag)
 		abbrevsize >>= 10;
 	}
 	len += snprintf(buf + len, sizeof(buf) - len,
-	    " %5qd %c%c ", (long long)abbrevsize, prefixes[i],
+	    " %5jd %c%c ", (intmax_t)abbrevsize, prefixes[i],
 	    prefixes[i] == ' ' ? ' ' : 'B');
 
 	timersub(&now, &lastupdate, &wait);
@@ -755,7 +757,7 @@ ptransfer(siginfo)
 	if (!verbose && !siginfo)
 		return;
 
-	(void)gettimeofday(&now, (struct timezone *)0);
+	(void)gettimeofday(&now, NULL);
 	timersub(&now, &start, &td);
 	elapsed = td.tv_sec + (td.tv_usec / 1000000.0);
 	bs = bytes / (elapsed == 0.0 ? 1 : elapsed);
@@ -764,8 +766,8 @@ ptransfer(siginfo)
 		meg = 1;
 	len = 0;
 	len += snprintf(buf + len, sizeof(buf) - len,
-	    "%qd byte%s %s in %.2f seconds (%.2f %sB/s)\n",
-	    (long long)bytes, bytes == 1 ? "" : "s", direction, elapsed,
+	    "%jd byte%s %s in %.2f seconds (%.2f %sB/s)\n",
+	    (intmax_t)bytes, bytes == 1 ? "" : "s", direction, elapsed,
 	    bs / (1024.0 * (meg ? 1024.0 : 1.0)), meg ? "M" : "K");
 	if (siginfo && bytes > 0 && elapsed > 0.0 && filesize >= 0
 	    && bytes + restart_point <= filesize) {
@@ -927,83 +929,83 @@ isipv6addr(const char *addr)
 int 
 auth()
 {
-    int n = 0, rcode = 0;
-    char *ssl_version;
-    int ssl_bits;
-    SSL_CIPHER *ssl_cipher;
+	int n = 0, rcode = 0;
+	char *ssl_version;
+	int ssl_bits;
+	SSL_CIPHER *ssl_cipher;
 
-    switch (ssl_secure_flags & SSL_ENABLED) {
-    case SSL_USE_TLS:
-        n = command("AUTH TLS");
-        if (n == COMPLETE) {
-	    break;
-	} else {
-	    printf("TLS not available\n");
-	    break;
+	switch (ssl_secure_flags & SSL_ENABLED) {
+	case SSL_USE_TLS:
+		n = command("AUTH TLS");
+		if (n == COMPLETE) {
+			break;
+		} else {
+			printf("TLS not available\n");
+			break;
+		}
+	case SSL_ENABLED:
+		n = command("AUTH TLS");
+		if (n == COMPLETE) break;
+	case SSL_USE_COMPAT:
+		n = command("AUTH SSL");
+		if (n == COMPLETE || n == CONTINUE) {
+			ssl_compat_flag = 1;
+			ssl_encrypt_data = 1;
+			/* It's a some sort of a workaround for the problem of
+			 * the reply code: 234 in FTP-TLS draft v7 and RFC2228
+			 * vs 334 in some early versions of the draft. */
+			n = COMPLETE; 
+			break;
+		}
+	default:
+		printf("TLS/SSL not available\n");
 	}
-    case SSL_ENABLED:
-	n = command("AUTH TLS");
-	if (n == COMPLETE) break;
-    case SSL_USE_COMPAT:
-	n = command("AUTH SSL");
-	if (n == COMPLETE || n == CONTINUE) {
-	    ssl_compat_flag=1;
-	    ssl_encrypt_data=1;
-	    /* some sort of a workaround for the problem of the reply code:
-	     * 234 in FTP-TLS draft v7 and RFC2228 vs 334 in some
-	     * early versions of the draft
-	     */
-	    n = COMPLETE; 
-	    break;
+
+	if (n == COMPLETE) {
+		rcode = 1;
+		/* Do SSL */
+		ssl_con = (SSL *)SSL_new(ssl_ctx);
+		SSL_set_connect_state(ssl_con);
+
+		SSL_set_fd(ssl_con, fileno(cout));
+		set_ssl_trace(ssl_con);
+
+		if ((n = SSL_connect(ssl_con)) <= 0) {
+			ssl_log_err(bio_err, "SSL_connect error");
+
+			switch (SSL_get_verify_result(ssl_con)) {
+			case X509_V_ERR_CRL_SIGNATURE_FAILURE:
+				ssl_log_msgn(bio_err, "Reason: Invalid signature on CRL!");
+				break;
+			case X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD:
+				ssl_log_msgn(bio_err, "Reason: Found CRL has invalid nextUpdate field.");
+				break;
+			case X509_V_ERR_CRL_HAS_EXPIRED:
+				ssl_log_msgn(bio_err, "Reason: Found CRL expired - revoking all certificates until you get updated CRL.");
+				break;
+			case X509_V_ERR_CERT_REVOKED:
+				ssl_log_msgn(bio_err, "Reason: Certificate is revoked");
+				break;
+			default:
+				break;
+			}
+
+			warnx("TLS/SSL connection to server failed");
+			/* abort time methinks ... */
+			/* exit(1);*/
+			rcode = -1;
+		} else {
+			ssl_version = SSL_get_cipher_version(ssl_con);
+			ssl_cipher = SSL_get_current_cipher(ssl_con);
+			SSL_CIPHER_get_bits(ssl_cipher, &ssl_bits);
+			fprintf(stderr, "[%s, cipher %s, %d bits]\n",
+			    ssl_version, SSL_CIPHER_get_name(ssl_cipher),
+			    ssl_bits);
+			fflush(stderr);
+			ssl_active_flag = 1;
+		}
 	}
-    default:
-	printf("TLS/SSL not available\n");
-    }
 
-    if (n == COMPLETE) {
-        rcode = 1;
-        /* do SSL */
-        ssl_con=(SSL *)SSL_new(ssl_ctx);
-        SSL_set_connect_state(ssl_con);
-
-        SSL_set_fd(ssl_con,fileno(cout));
-        set_ssl_trace(ssl_con);
-
-	if ((n=SSL_connect(ssl_con))<=0) {
-	    ssl_log_err(bio_err, "SSL_connect error");
-
-	    switch (SSL_get_verify_result(ssl_con)) {
-	    case X509_V_ERR_CRL_SIGNATURE_FAILURE:
-		ssl_log_msgn(bio_err, "Reason: Invalid signature on CRL!");
-		break;
-	    case X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD:
-		ssl_log_msgn(bio_err, "Reason: Found CRL has invalid nextUpdate field.");
-		break;
-	    case X509_V_ERR_CRL_HAS_EXPIRED:
-		ssl_log_msgn(bio_err,
-		    "Reason: Found CRL is expired - revoking all certificates until you get updated CRL.");
-		break;
-	    case X509_V_ERR_CERT_REVOKED:
-		ssl_log_msgn(bio_err,"Reason: Certificate revoked");
-		break;
-	    default:
-	    }
-
-	    warnx("TLS/SSL connection to server failed");
-	    /* abort time methinks ... */
-	    /* exit(1);*/
-	    rcode = -1;
-	} else {
-	    ssl_version=SSL_get_cipher_version(ssl_con);
-	    ssl_cipher=SSL_get_current_cipher(ssl_con);
-	    SSL_CIPHER_get_bits(ssl_cipher, &ssl_bits);
-	    fprintf(stderr,"[%s, cipher %s, %d bits]\n",ssl_version,
-		    SSL_CIPHER_get_name(ssl_cipher), ssl_bits);
-	    fflush(stderr);
-	    ssl_active_flag=1;
-	}
-    }
-
-    return(rcode);
+	return(rcode);
 }
 #endif /* USE_SSL */
