@@ -1,6 +1,6 @@
 /* sslapp.c	- ssl application code */
 /*-
- * Copyright (c) 2002, 2003 Nick Leuta
+ * Copyright (c) 2002, 2003, 2004, 2005 Nick Leuta
  * All rights reserved.
  *
  * This software is based on code developed by Tim Hudson <tjh@cryptsoft.com>
@@ -37,6 +37,7 @@
 #include <signal.h>
 #include <stdarg.h>
 #include <string.h>
+#include <syslog.h>
 #include <unistd.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -47,195 +48,189 @@
 #include <sys/time.h>
 #endif
 
+#include <port_base.h>
+
 #include "sslapp.h"
 
-BIO *bio_err;
-SSL *ssl_con;
-SSL_CTX *ssl_ctx;
-X509_STORE *x509st_CRL = NULL;
+BIO	*bio_err;
+SSL	*ssl_con;
+SSL_CTX	*ssl_ctx;
+X509_STORE	*x509st_CRL = NULL;
 
-int ssl_debug_flag=0;
-int ssl_active_flag=0;
-int ssl_verify_flag=SSL_VERIFY_NONE;
-int ssl_verbose_flag=0;
-char *ssl_cert_file=NULL;
-char *ssl_key_file=NULL;
-char *ssl_cipher_list=NULL;
-char *ssl_log_file=NULL;
-char *ssl_CA_file=NULL;
-char *ssl_CA_path=NULL;
-char *ssl_CRL_file=NULL;
-char *ssl_CRL_path=NULL;
+int	ssl_debug_flag = 0;
+int	ssl_active_flag = 0;
+int	ssl_verify_flag = SSL_VERIFY_NONE;
+int	ssl_logerr_syslog = 0;	/* Log error information to syslog */
+int	ssl_verbose_flag = 0;
+char	*ssl_cert_file = NULL;
+char	*ssl_key_file = NULL;
+char	*ssl_cipher_list = NULL;
+char	*ssl_log_file = NULL;
+char	*ssl_CA_file = NULL;
+char	*ssl_CA_path = NULL;
+char	*ssl_CRL_file = NULL;
+char	*ssl_CRL_path = NULL;
 
 /* fwd decl */
 static void client_info_callback();
 
-int do_ssleay_init(server)
-int server;
+int
+do_ssleay_init(int server)
 {
-  char *p;
+    char *p;
 
-  /* make sure we have somewhere we can log errors to */
-  if (bio_err==NULL) {
-    if ((bio_err=BIO_new(BIO_s_file()))!=NULL) {
-      if (ssl_log_file==NULL)
-	BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
-      else {
-	if (BIO_write_filename(bio_err,ssl_log_file)<=0) {
-	  /* not a lot we can do */
-	}
-      }
+    /* Make sure we have somewhere we can log errors to */
+    if (bio_err == NULL) {
+	    if ((bio_err = BIO_new(BIO_s_file())) != NULL) {
+		    if (ssl_log_file == NULL) {
+			    BIO_set_fp(bio_err, stderr, BIO_NOCLOSE);
+		    } else {
+			    if (BIO_write_filename(bio_err, ssl_log_file)<=0) {
+			    /* not a lot we can do */
+			    }
+		    }
+	    }
     }
-  }
 
-  /* rather simple things these days ... the old SSL_LOG and SSL_ERR
-   * vars are long gone now SSLeay8 has rolled around and we have 
-   * a clean interface for doing things
-   */
-  if (ssl_debug_flag)
-    ssl_log_msgn(bio_err, "SSL_DEBUG_FLAG on");
+    if (ssl_debug_flag)
+	    ssl_log_msgn(bio_err, "SSL_DEBUG_FLAG on");
 
-  /* init things so we will get meaningful error messages
-   * rather than numbers 
-   */
-  SSL_load_error_strings();
+    /*
+     * Init things so we will get meaningful error messages rather than
+     * numbers
+     */
+    SSL_load_error_strings();
+    SSLeay_add_ssl_algorithms();
+    ssl_ctx = (SSL_CTX *)SSL_CTX_new(SSLv23_method());
 
-#ifdef SSLEAY8
-  SSLeay_add_ssl_algorithms();
-  ssl_ctx=(SSL_CTX *)SSL_CTX_new(SSLv23_method());
+    /*
+     * We may require a temp 512 bit RSA key because of the wonderful way
+     * export things work... If so we generate one now!
+     */
+    if (server) {
+	    const char ctx_sid[] = "BSDftpd-ssl";
+	    SSL_CTX_set_session_id_context(ssl_ctx, ctx_sid, strlen(ctx_sid));
 
-  /* we may require a temp 512 bit RSA key because of the
-   * wonderful way export things work ... if so we generate
-   * one now!
-   */
-  if (server) {
-    const char ctx_sid[]="BSDftpd-ssl";
-    SSL_CTX_set_session_id_context(ssl_ctx, ctx_sid, strlen(ctx_sid));
+	    if (SSL_CTX_need_tmp_RSA(ssl_ctx)) {
+		    RSA *rsa;
 
-    if (SSL_CTX_need_tmp_RSA(ssl_ctx)) {
-      RSA *rsa;
+		    if (ssl_debug_flag)
+			    ssl_log_msgn(bio_err, "Generating temp (512 bit) RSA key...");
+		    rsa = RSA_generate_key(512, RSA_F4, NULL, NULL);
+		    if (ssl_debug_flag)
+			    ssl_log_msgn(bio_err, "Generation of temp (512 bit) RSA key done");
 
-      if (ssl_debug_flag)
-	  ssl_log_msgn(bio_err, "Generating temp (512 bit) RSA key ...");
-      rsa=RSA_generate_key(512,RSA_F4,NULL,NULL);
-      if (ssl_debug_flag)
-	  ssl_log_msgn(bio_err, "Generation of temp (512 bit) RSA key done");
-   
-      if (!SSL_CTX_set_tmp_rsa(ssl_ctx,rsa)) {
-	  ssl_log_msgn(bio_err, "Failed to assign generated temp RSA key!");
-      }
-
-      RSA_free(rsa);
-      if (ssl_debug_flag)
-	  ssl_log_msgn(bio_err, "Assigned temp (512 bit) RSA key");
+		    if (!SSL_CTX_set_tmp_rsa(ssl_ctx, rsa)) {
+			    ssl_log_msgn(bio_err, "Failed to assign generated temp RSA key!");
+		    }
+		    RSA_free(rsa);
+		    if (ssl_debug_flag)
+			    ssl_log_msgn(bio_err, "Assigned temp (512 bit) RSA key");
+	    }
     }
-  }
 
-  /* also switch on all the interoperability and bug
-   * workarounds so that we will communicate with people
-   * that cannot read poorly written specs :-)
-   */
-  SSL_CTX_set_options(ssl_ctx,SSL_OP_ALL);
+    /*
+     * Also switch on all the interoperability and bug workarounds so
+     * that we will communicate with people that cannot read poorly
+     * written specs :-)
+     */
+    SSL_CTX_set_options(ssl_ctx, SSL_OP_ALL);
 
-#else /* !SSLEAY8 */
-  ssl_ctx=(SSL_CTX *)SSL_CTX_new();
-#endif /* SSLEAY8 */
+    /* The user can set whatever ciphers they want to use */
+    if (ssl_cipher_list == NULL) {
+	    p = getenv("SSL_CIPHER");
+	    if (p != NULL)
+		    SSL_CTX_set_cipher_list(ssl_ctx, p);
+    } else
+	    SSL_CTX_set_cipher_list(ssl_ctx, ssl_cipher_list);
 
-  /* the user can set whatever ciphers they want to use */
-  if (ssl_cipher_list==NULL) {
-      p=getenv("SSL_CIPHER");
-      if (p!=NULL)
-        SSL_CTX_set_cipher_list(ssl_ctx,p);
-  } else
-      SSL_CTX_set_cipher_list(ssl_ctx,ssl_cipher_list);
+    /*
+     * For verbose we use the 0.6.x info callback that I got eric to
+     * finally add into the code :-) --tjh
+     */
+    if (ssl_verbose_flag)
+	    SSL_CTX_set_info_callback(ssl_ctx, client_info_callback);
 
-  /* for verbose we use the 0.6.x info callback that I got
-   * eric to finally add into the code :-) --tjh
-   */
-  if (ssl_verbose_flag) {
-      SSL_CTX_set_info_callback(ssl_ctx,client_info_callback);
-  }
+    /* Add in any certificates if you want to here... */
+    if (ssl_cert_file) {
+	    if (!SSL_CTX_use_certificate_file(ssl_ctx, ssl_cert_file,
+		X509_FILETYPE_PEM)) {
+		    ssl_log_err(bio_err, "Error loading '%s'", ssl_cert_file);
+		    return (0);
+	    } else {
+		    if (!ssl_key_file)
+			    ssl_key_file = ssl_cert_file;
+			    if (!SSL_CTX_use_RSAPrivateKey_file(ssl_ctx,
+				ssl_key_file, X509_FILETYPE_PEM)) {
+				    ssl_log_err(bio_err, "Error loading '%s'",
+					ssl_cert_file);
+				    return (0);
+			    }
+	    }
+    }
 
-  /* Add in any certificates if you want to here ... */
-  if (ssl_cert_file) {
-      if (!SSL_CTX_use_certificate_file(ssl_ctx, ssl_cert_file, 
-		      X509_FILETYPE_PEM)) {
-	  ssl_log_err(bio_err, "Error loading %s",ssl_cert_file);
-	  return(0);
-      } else {
-	  if (!ssl_key_file)
-	      ssl_key_file = ssl_cert_file;
-	  if (!SSL_CTX_use_RSAPrivateKey_file(ssl_ctx, ssl_key_file,
-		      X509_FILETYPE_PEM)) {
-	      ssl_log_err(bio_err, "Error loading %s",ssl_cert_file);
-	      return(0);
-	  }
-      }
-  }
+    /*
+     * Check if certificate locations specified in command-line or
+     * environment. Command-line values will override environment ones.
+     */
+    if (ssl_CA_file != NULL || ssl_CA_path != NULL ||
+	getenv(X509_get_default_cert_file_env()) ||
+	getenv(X509_get_default_cert_dir_env())) {
+		SSL_CTX_load_verify_locations(ssl_ctx,
+		    ssl_CA_file ? ssl_CA_file : getenv(X509_get_default_cert_file_env()),
+		    ssl_CA_path ? ssl_CA_path : getenv(X509_get_default_cert_dir_env()));
+    } else {
+	    /*
+	     * Make sure we will find certificates in the standard
+	     * location ... Otherwise we don't look anywhere for these
+	     * things which is going to make client certificate exchange
+	     * rather useless :-)
+	     */
+	    SSL_CTX_set_default_verify_paths(ssl_ctx);
+    }
 
-#ifdef SSLEAY8
-  /* Check if certificate locations specified in command-line or environment.
-   * Command-line values will override environment ones.
-   */
-  if ( ssl_CA_file!=NULL || ssl_CA_path!=NULL ||
-         getenv(X509_get_default_cert_file_env()) ||
-         getenv(X509_get_default_cert_dir_env()) ) {
-      SSL_CTX_load_verify_locations(ssl_ctx, 
-        ssl_CA_file ? ssl_CA_file : getenv(X509_get_default_cert_file_env()),
-	ssl_CA_path ? ssl_CA_path : getenv(X509_get_default_cert_dir_env()));
-  } else
-      /* make sure we will find certificates in the standard
-       * location ... otherwise we don't look anywhere for
-       * these things which is going to make client certificate
-       * exchange rather useless :-)
-       */
-      SSL_CTX_set_default_verify_paths(ssl_ctx);
+    /*
+     * Check if CRL locations specified in command-line or environment.
+     * Command-line values will override environment ones.
+     */
+    if (ssl_CRL_file != NULL || ssl_CRL_path != NULL ||
+	getenv("SSL_CRL_FILE") || getenv("SSL_CRL_DIR")) {
+		x509st_CRL = ssl_X509_STORE_create(
+		    ssl_CRL_file ? ssl_CRL_file : getenv("SSL_CRL_FILE"),
+		    ssl_CRL_path ? ssl_CRL_path : getenv("SSL_CRL_DIR"));
+    } else {
+	    /*
+	     * Set defaults for CRL locations
+	     */
+	    char ssl_CRL_file_tmp[MAXPATHLEN], ssl_CRL_path_tmp[MAXPATHLEN];
+	    snprintf(ssl_CRL_file_tmp, sizeof(ssl_CRL_file_tmp), "%s/crl.pem",
+		X509_get_default_cert_area());
+	    snprintf(ssl_CRL_path_tmp, sizeof(ssl_CRL_path_tmp), "%s",
+		X509_get_default_cert_dir());
+	    x509st_CRL = ssl_X509_STORE_create(ssl_CRL_file_tmp,
+		ssl_CRL_path_tmp);
+    }
 
-  /* Check if CRL locations specified in command-line or environment.
-   * Command-line values will override environment ones.
-   */
-  if ( ssl_CRL_file!=NULL || ssl_CRL_path!=NULL ||
-         getenv("SSL_CRL_FILE") || getenv("SSL_CRL_DIR") ) {
-      x509st_CRL = ssl_X509_STORE_create(
-        ssl_CRL_file ? ssl_CRL_file : getenv("SSL_CRL_FILE"),
-	ssl_CRL_path ? ssl_CRL_path : getenv("SSL_CRL_DIR"));
-  } else {
-      /* Setting defaults for CRL locations
-       */
-      static char ssl_CRL_file_tmp[MAXPATHLEN], ssl_CRL_path_tmp[MAXPATHLEN];
-      snprintf(ssl_CRL_file_tmp, sizeof(ssl_CRL_file_tmp), "%s/crl.pem",
-    		X509_get_default_cert_area());
-      snprintf(ssl_CRL_path_tmp, sizeof(ssl_CRL_path_tmp), "%s",
-    		X509_get_default_cert_dir());
-      x509st_CRL = ssl_X509_STORE_create(ssl_CRL_file_tmp,ssl_CRL_path_tmp);
-  }
-#else
-  SSL_set_default_verify_paths(ssl_ctx);
-#endif
+    SSL_CTX_set_verify(ssl_ctx, ssl_verify_flag, verify_cb_CRL);
 
-  SSL_CTX_set_verify(ssl_ctx,ssl_verify_flag,verify_cb_CRL);
-
-  return(1);
+    return (1);
 }
 
-
-static void client_info_callback(s,where,ret)
-SSL *s;
-int where;
-int ret;
+static void
+client_info_callback(SSL *s, int where, int ret)
 {
-  if (where==SSL_CB_CONNECT_LOOP) {
-    ssl_log_msgn(bio_err,"SSL_connect:%s %s",
-		SSL_state_string(s),SSL_state_string_long(s));
-  } else if (where==SSL_CB_CONNECT_EXIT) {
-    if (ret == 0) {
-      ssl_log_msgn(bio_err,"SSL_connect:failed in %s %s",
-		    SSL_state_string(s),SSL_state_string_long(s));
-    } else if (ret < 0) {
-      ssl_log_msgn(bio_err,"SSL_connect:error in %s %s",
-		    SSL_state_string(s),SSL_state_string_long(s));
+    if (where == SSL_CB_CONNECT_LOOP) {
+	ssl_log_msgn(bio_err, "SSL_connect:%s %s",
+	    SSL_state_string(s), SSL_state_string_long(s));
+    } else if (where == SSL_CB_CONNECT_EXIT) {
+	if (ret == 0) {
+	    ssl_log_msgn(bio_err, "SSL_connect:failed in %s %s",
+		SSL_state_string(s), SSL_state_string_long(s));
+	} else if (ret < 0) {
+	    ssl_log_msgn(bio_err, "SSL_connect:error in %s %s",
+		SSL_state_string(s), SSL_state_string_long(s));
+	}
     }
-  }
 }
 
 /*
@@ -442,8 +437,8 @@ ssl_X509_cmp(X509 *x509_cert1, X509 *x509_cert2)
 {
     /* X509_cmp() will crash if any of its args are NULL
      */
-    if (x509_cert1!=NULL) {
-	if (x509_cert2==NULL) {
+    if (x509_cert1 != NULL) {
+	if (x509_cert2 == NULL) {
 	    return -3; /* x509_cert1 in not NULL, x509_cert2 is NULL */
 	} else {
 	    if (X509_cmp(x509_cert1, x509_cert2)) {
@@ -453,7 +448,7 @@ ssl_X509_cmp(X509 *x509_cert1, X509 *x509_cert2)
 	    }
 	}
     } else {
-	if (x509_cert2==NULL) {
+	if (x509_cert2 == NULL) {
 	    return -1; /* both certificates are NULL */
 	} else {
 	    return -2; /* x509_cert1 is NULL, x509_cert2 is not NULL */
@@ -462,107 +457,158 @@ ssl_X509_cmp(X509 *x509_cert1, X509 *x509_cert2)
 }
 
 /*
- * Log an error and a debug information
+ * Log an error and a debug information.
  */
 
-/* Log the message
+/*
+ * Log the message.
  */
-void ssl_log_msg(BIO *bio, const char *fmt, ...)
+void
+ssl_log_msg(BIO *bio, const char *fmt, ...)
 {
-  va_list ap;
-  char outputbuf[2048];
+    va_list ap;
+    char *outputbuf;
 
-  va_start(ap,fmt);
+    va_start(ap, fmt);
+    vasprintf(&outputbuf, fmt, ap);
+    va_end(ap);
+    if (outputbuf == NULL) {
+	    BIO_printf(bio, "\r\nRan out of memory.\r\n");
+	    BIO_flush(bio);
+	    return;
+    }
 
-  (void)vsnprintf(outputbuf, sizeof(outputbuf), fmt, ap);
-  BIO_printf(bio, outputbuf);
-  BIO_flush(bio);
+    BIO_printf(bio, "%s", outputbuf);
+    BIO_flush(bio);
 
-  va_end(ap);
-}
-
-/* Log the message prepended and appended by the newline
- */
-void ssl_log_msgn(BIO *bio, const char *fmt, ...)
-{
-  va_list ap;
-  char outputbuf[2048];
-
-  va_start(ap,fmt);
-
-  (void)vsnprintf(outputbuf, sizeof(outputbuf), fmt, ap);
-  BIO_printf(bio, "\r\n%s\r\n", outputbuf);
-  BIO_flush(bio);
-
-  va_end(ap);
-}
-
-/* Log the message appended by the reason of the last error code from the SSL
- * error queue and removes that code.
- */
-void ssl_log_vwarn(BIO *bio, const char *fmt, va_list ap)
-{
-  char outputbuf[2048];
-
-  (void)vsnprintf(outputbuf, sizeof(outputbuf), fmt, ap);
-  BIO_printf(bio, "%s: %s\r\n", outputbuf,
-	    ERR_reason_error_string(ERR_get_error()));
-  BIO_flush(bio);
-}
-
-void ssl_log_warn(BIO *bio, const char *fmt, ...)
-{
-  va_list ap;
-
-  va_start(ap,fmt);
-  ssl_log_vwarn(bio, fmt, ap);
-  va_end(ap);
-}
-
-/* Log the message appended by the human-readable string that represents the
- * last error code from the SSL error queue and removes it.
- */
-void ssl_log_vwarn_debug(BIO *bio, const char *fmt, va_list ap)
-{
-  char outputbuf[2048];
-
-  (void)vsnprintf(outputbuf, sizeof(outputbuf), fmt, ap);
-  BIO_printf(bio, "%s: %s\r\n", outputbuf,
-	    ERR_error_string(ERR_get_error(),NULL));
-  BIO_flush(bio);
-}
-
-void ssl_log_warn_debug(BIO *bio, const char *fmt, ...)
-{
-  va_list ap;
-
-  va_start(ap,fmt);
-  ssl_log_vwarn_debug(bio, fmt, ap);
-  va_end(ap);
-}
-
-/* Log the message with the verbosity level depending on the SSL debug state
- */
-void ssl_log_err(BIO *bio, const char *fmt, ...)
-{
-  va_list ap;
-
-  va_start(ap,fmt);
-
-  if (ssl_debug_flag) {
-    ssl_log_vwarn_debug(bio, fmt, ap);
-  } else {
-    ssl_log_vwarn(bio, fmt, ap);
-  }
-
-  va_end(ap);
+    free(outputbuf);
 }
 
 /*
- * Support for the X.509 client authentication
+ * Log the message prepended and appended by the newline.
+ */
+void
+ssl_log_msgn(BIO *bio, const char *fmt, ...)
+{
+    va_list ap;
+    char *outputbuf;
+
+    va_start(ap, fmt);
+    vasprintf(&outputbuf, fmt, ap);
+    va_end(ap);
+    if (outputbuf == NULL) {
+	    BIO_printf(bio, "\r\nRan out of memory.\r\n");
+	    BIO_flush(bio);
+	    return;
+    }
+
+    BIO_printf(bio, "\r\n%s\r\n", outputbuf);
+    BIO_flush(bio);
+
+    free(outputbuf);
+}
+
+/*
+ * Common code for both ssl_log_vwarn() and ssl_log_vwarn_debug().
+ */
+void
+ssl_log_vwarn_common(BIO *bio, int debug_flag, const char *fmt, va_list ap)
+{
+    char *tmp, *outputbuf;
+
+    vasprintf(&tmp, fmt, ap);
+    if (tmp == NULL) {
+	    BIO_printf(bio, "\r\nRan out of memory.\r\n");
+	    BIO_flush(bio);
+	    if (ssl_logerr_syslog)
+		    syslog(LOG_ERR, "Ran out of memory.");
+	    return;
+    }
+
+    asprintf(&outputbuf, "%s: %s", tmp,
+	    debug_flag ? ERR_error_string(ERR_get_error(), NULL) :
+	    ERR_reason_error_string(ERR_get_error()));
+    free(tmp);
+    if (outputbuf == NULL) {
+	    BIO_printf(bio, "\r\nRan out of memory.\r\n");
+	    BIO_flush(bio);
+	    if (ssl_logerr_syslog)
+		    syslog(LOG_ERR, "Ran out of memory.");
+	    return;
+    }
+
+    BIO_printf(bio, "%s\r\n", outputbuf);
+    BIO_flush(bio);
+    if (ssl_logerr_syslog)
+	    syslog(LOG_WARNING, "%s", outputbuf);
+    free(outputbuf);
+}
+
+/*
+ * Log the message appended by the reason of the last error code from the SSL
+ * error queue and removes that code from the queue.
+ */
+void
+ssl_log_vwarn(BIO *bio, const char *fmt, va_list ap)
+{
+    ssl_log_vwarn_common(bio, 0, fmt, ap);
+}
+
+void
+ssl_log_warn(BIO *bio, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap,fmt);
+    ssl_log_vwarn(bio, fmt, ap);
+    va_end(ap);
+}
+
+/*
+ * Log the message appended by the human-readable string that represents the
+ * last error code from the SSL error queue and removes it.
+ */
+void
+ssl_log_vwarn_debug(BIO *bio, const char *fmt, va_list ap)
+{
+    ssl_log_vwarn_common(bio, 1, fmt, ap);
+}
+
+void
+ssl_log_warn_debug(BIO *bio, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap,fmt);
+    ssl_log_vwarn_debug(bio, fmt, ap);
+    va_end(ap);
+}
+
+/*
+ * Log the message with the verbosity level depending on the SSL debug state.
+ */
+void
+ssl_log_err(BIO *bio, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+
+    if (ssl_debug_flag) {
+	    ssl_log_vwarn_debug(bio, fmt, ap);
+    } else {
+	    ssl_log_vwarn(bio, fmt, ap);
+    }
+
+    va_end(ap);
+}
+
+/*
+ * Support for the X.509 client authentication.
  */
 
-/* Get the text value by the object type.
+/*
+ * Get the text value by the object type.
  * arguments:
  *	obj_type - object type name, may be provided in "short" (SN; for
  *	           example "CN") or "long" (LN; for example "commonName")
@@ -580,7 +626,7 @@ void ssl_log_err(BIO *bio, const char *fmt, ...)
  */
 int
 ssl_X509_NAME_get_text_by_name(X509_NAME *x509_name, char *obj_type,
-			    char *str, int len)
+    char *str, int len)
 {
     int nid;
     
@@ -590,7 +636,8 @@ ssl_X509_NAME_get_text_by_name(X509_NAME *x509_name, char *obj_type,
     return (X509_NAME_get_text_by_NID(x509_name, nid, str, len));
 }
 
-/* Get the login name from the certificate.
+/*
+ * Get the login name from the certificate.
  * arguments:
  *	cert     - X.509 certificate.
  *	obj_type - object type name, may be provided in "short" (SN; for
@@ -616,14 +663,14 @@ x509_get_value(X509 *cert, char *obj_type, char *username, int len)
 	ret = ssl_X509_NAME_get_text_by_name(x509_subject_name, obj_type,
 					    username, len);
 
-    if (ret > 0) {
+    if (ret > 0)
 	return 1;
-    } else {
+    else
 	return 0;
-    }
 }
 
-/* Get the login name by the e-mail address from the certificate.
+/*
+ * Get the login name by the e-mail address from the certificate.
  * arguments:
  *	cert     - X.509 certificate.
  *	obj_type - object type name, may be provided in "short" (SN; for
@@ -650,21 +697,21 @@ ssl_get_login_by_email(X509 *cert, char *obj_type, char *domain,
 {
     char email[512], *cp_login, *cp_domain;
 
-    /* extract the field from the cert */
+    /* Extract the field from the cert */
     if (x509_get_value(cert, obj_type, email, sizeof(email)) > 0) {
-	/* process e-mail */
+	/* Process e-mail */
 	cp_login = email;
 	cp_domain = strchr(email, '@');
 	if (!cp_domain) {
-	    /* invalid e-mail address */
+	    /* Invalid e-mail address */
 	    return 0;
 	}
 	*cp_domain++ = '\0';
 
-	/* if domain is specified, it must be compared with the cert's one */
+	/* If domain is specified, it must be compared with the cert's one */
 	if (domain != NULL) {
 	    if (strcasecmp(cp_domain, domain) != 0) {
-		/* cert's e-mail domain doesn't match the expected one */
+		/* Cert's e-mail domain doesn't match the expected one */
 		return 0;
 	    }
 	}
@@ -673,14 +720,15 @@ ssl_get_login_by_email(X509 *cert, char *obj_type, char *domain,
 	return 1;
     }
 
-    /* error extracting a field from the cert */
+    /* Error extracting a field from the cert */
     if (ssl_debug_flag)
 	ssl_log_msgn(bio_err,
 	    "ssl_get_login_by_email(): error extracting field: %s", obj_type);
     return 0;
 }
 
-/* Check if the client's certificate is presented in the specified file.
+/*
+ * Check if the client's certificate is presented in the specified file.
  * arguments:
  *	cert     - client's X.509 certificate.
  *	filename - name of the file that contains a set of certificates.
@@ -706,15 +754,15 @@ ssl_check_cert_file(X509 *cert, char *filename)
     if (lstat(filename, &stbuf) < 0 || !S_ISREG(stbuf.st_mode)) {
 	if (ssl_debug_flag)
 	    ssl_log_msgn(bio_err,
-	    "ssl_check_cert_file(): %s is not a plain file or is a symlink",
-	    filename);
+		"ssl_check_cert_file(): '%s' is not a plain file or is a symlink",
+		filename);
     	return 0;
     }
 
     if (!(fp = fopen(filename, "r"))) {
 	if (ssl_debug_flag)
 	    ssl_log_msgn(bio_err,
-		"ssl_check_cert_file(): can't open %s for reading", filename);
+		"ssl_check_cert_file(): can't open '%s' for reading", filename);
     	return 0;
     }
 
@@ -753,7 +801,7 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
     FILE *f_in = NULL;     /* stream for child's stdin */
     pid_t child_pid = -1;
     int oldmask, status;
-    char buf[1024], *cp_rcode, *cp_name, *str;
+    char buf[BUFSIZ], *cp_rcode, *cp_name, *str;
     fd_set rfds;
     struct timeval tv;
 
@@ -774,7 +822,7 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
     }
 
     /*
-     * fork
+     * Fork
      */
     child_pid = fork();
     /* Is it the parent or the child process? */
@@ -783,20 +831,20 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
 	ret = 0;
 	goto end;
     case  0: /* In the child */
-	/* map stdin/stdout/stderr to the pipe to the parent */
+	/* Map stdin/stdout/stderr to the pipe to the parent */
         dup2(p_in[0], 0);  /* stdin */
         dup2(p_out[1], 1); /* stdout */
 	dup2(p_out[1], 2); /* stderr */
 
-	/* close descriptors of pipes */
+	/* Close descriptors of pipes */
         close(p_in[0]);
         close(p_in[1]);
         close(p_out[1]);
         close(p_out[0]);
 
-	/* exec */
+	/* Exec */
         execl(progname, progname, NULL);
-	/* exec failed */
+	/* Exec failed */
         _exit(1);
     }
 
@@ -804,22 +852,22 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
      * In the parent process
      */
 
-    /* those ends of pipes must not be used by the parent */
+    /* Those ends of pipes must not be used by the parent */
     close(p_in[0]);
     close(p_out[1]);
 
-    /* get the control over child's stdin */
+    /* Get the control over child's stdin */
     if (!(f_in = fdopen(p_in[1], "w"))) {
 	ret = 0;
 	goto end;
     }
 
-    /* send tokens to the auth program */
+    /* Send tokens to the auth program */
     fprintf(f_in, "%s\r\n", name);
     PEM_write_X509(f_in, cert);
     fflush(f_in);
 
-    /* wait for the result of the authentication from child's stdout */
+    /* Wait for the result of the authentication from child's stdout */
     FD_ZERO(&rfds);
     FD_SET(p_out[0], &rfds);
 
@@ -832,12 +880,12 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
 	ret = 0;
 	if (ssl_debug_flag)
 	    ssl_log_msgn(bio_err,
-		    "No response from the external authentication program");
+		"No response from the external authentication program");
 	goto end;
     }
 
-    /* read the result of the authentication */
-    err = read(p_out[0], buf, sizeof(buf)-1);
+    /* Read the result of the authentication */
+    err = read(p_out[0], buf, sizeof(buf) - 1);
     if (err <= 7) { /* error reading the reply of the authenticator, or an
 		     * incorrect reply (min length == 7) */
 	ret = 0;
@@ -858,7 +906,7 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
 	ret = 0;
 	goto end;
     }
-    /* wipe cr/lf symbols */
+    /* Wipe cr/lf symbols */
     str = strchr(cp_rcode, '\r');
     if (str)
 	*str = '\0';
@@ -869,13 +917,13 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
     if (str)
 	*str = '\0';
 
-    /* process the result of the authentication */
+    /* Process the result of the authentication */
     if (cp_rcode[0] == '1') { /* 1xx - access allowed by the program */
 	if (strlen(name) > 0) {
-	    /* sanity checking */
+	    /* Sanity checking */
 	    ret = (strcmp(name, cp_name) == 0) ? 1 : 0;
 	} else {
-	    /* return the login name extracted from the certificate */
+	    /* Return the login name extracted from the certificate */
 	    strncpy(name, cp_name, len);
 	    ret = 1;
 	}
@@ -884,18 +932,18 @@ ssl_check_ext_prog(X509 *cert, char *name, int len, char *progname)
     }
 
 end:
-    /* close streams */
+    /* Close streams */
     if (f_in!=NULL)
 	fclose(f_in);
 
-    /* close used ends of pipes */
+    /* Close used ends of pipes */
     close(p_in[1]);
     close(p_out[0]);
-    /* close other ends of pipes, normally it isn't needed */
+    /* Close other ends of pipes, normally it isn't needed */
     close(p_in[0]);
     close(p_out[1]);
 
-    /* wait for child's termination */
+    /* Wait for child's termination */
     if (child_pid > 0) {
 	oldmask = sigblock(sigmask(SIGINT)|sigmask(SIGQUIT)|sigmask(SIGHUP));
 	while (waitpid(child_pid, &status, 0) < 0 && errno == EINTR)
@@ -925,25 +973,25 @@ int
 ssl_x509_auth(char *service, X509 *cert, char *name)
 {
     FILE *user_fp;
-    char buf[4096];
+    char buf[_X509_AUTH_MAXSTRLEN];
     int ret = 1; /* 0 - all ok, !=0 - something wrong */
 
-    char username[256];
+    char username[MAXLOGNAME];
     char *ssl_auth_name = NULL;
 
-    /* sanity checking */
+    /* Sanity checking */
     if (service == NULL) {
-	/* the service name isn't provided */
+	/* The service name isn't provided */
 	if (ssl_debug_flag)
 	    ssl_log_msgn(bio_err, "ssl_x509_auth(): service name is NULL");
 	return -6;
     }
 
     if (cert != NULL) {
-	/* get the certificate subject */
+	/* Get the certificate subject */
 	ssl_auth_name = (char*)ONELINE_NAME(X509_get_subject_name(cert));
     } else {
-	/* can't authenticate because no peer certificate provided */
+	/* Can't authenticate because no peer certificate provided */
 	if (ssl_debug_flag)
 	    ssl_log_msgn(bio_err, "ssl_x509_auth(): certificate is NULL");
 	return -5;
@@ -952,21 +1000,21 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
     user_fp = fopen(_PATH_X509_AUTH, "r");
     if (!user_fp) {
 	if (ssl_debug_flag)
-	    ssl_log_msgn(bio_err, "ssl_x509_auth(): can't open %s for reading",
-	                 _PATH_X509_AUTH);
+	    ssl_log_msgn(bio_err,
+		"ssl_x509_auth(): can't open '%s' for reading", _PATH_X509_AUTH);
 	return -7;
     }
 
-    while (fgets(buf, sizeof buf, user_fp)) {
+    while (fgets(buf, sizeof(buf), user_fp)) {
 	char *cp_service, *cp_action, *cp_userlist, *cp_subject;
 	char *n;
 	int action_flag; /* deny (0) | allow (1) */
 
-	/* allow comments in the file */
+	/* Allow comments in the file */
 	if (buf[0]=='#')
 	    continue;
 
-	/* wipe cr/lf symbols */
+	/* Wipe cr/lf symbols */
 	n = strchr(buf, '\n');
 	if (n)
 	    *n = '\0';
@@ -1014,8 +1062,8 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
 		break;
 	    }
 	default:
-	    ssl_log_msgn(bio_err, "%s: invalid action \"%s\"", _PATH_X509_AUTH,
-	                 cp_action);
+	    ssl_log_msgn(bio_err, "%s: invalid action: %s", _PATH_X509_AUTH,
+		cp_action);
 	    continue;
 	}
 
@@ -1040,22 +1088,22 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
 	    case '/': /* username is a field of provided subject */
 		if (n[1] == '/') {
 	    	    /* The field contains the e-mail address */
-	    	    char *domain, field[256];
+	    	    char *domain, field[BUFSIZ];
 
-		    domain = strchr(n+2, '/');
+		    domain = strchr(n + 2, '/');
 		    if (domain)
 			*domain++ = '\0';
-		    strncpy(field, n+2, sizeof(field));
+		    strncpy(field, n + 2, sizeof(field));
 
 		    if (ssl_get_login_by_email(cert, field, domain, username,
-			    sizeof(username)) <=0) {
+			    sizeof(username)) <= 0) {
 			n = cp_userlist;
 			continue;
 		    }
 		} else {
 	    	    /* The field contains the username itself */
-		    if (x509_get_value(cert, n+1, username, 
-			    sizeof(username)) <=0) {
+		    if (x509_get_value(cert, n + 1, username, 
+			    sizeof(username)) <= 0) {
 			n = cp_userlist;
 			continue;
 		    }
@@ -1084,23 +1132,23 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
 		switch (cp_subject[1]) {
 		case 'r': { /* subject is defined as a reqular expression */
 		    regex_t preg;
-		    cp_subject = cp_subject+2;
+		    cp_subject = cp_subject + 2;
 
     		    regcomp(&preg, cp_subject, REG_EXTENDED|REG_NOSUB);
-    		    ret=regexec(&preg, ssl_auth_name, 0, 0, 0);
+    		    ret = regexec(&preg, ssl_auth_name, 0, 0, 0);
     		    regfree(&preg);
 		    break;
 		}
 		case 'f': { /* certificate itself in a file */
 		    struct passwd *pwd;
-		    char fnbuf[MAXPATHLEN+1];
-		    cp_subject = cp_subject+2;
+		    char fnbuf[MAXPATHLEN + 1];
+		    cp_subject = cp_subject + 2;
 
-		    /* do '~' expansion */
+		    /* Do '~' expansion */
 		    if (cp_subject[0] == '~') {
 			char *cp;
 
-			cp = cp_subject+1;
+			cp = cp_subject + 1;
 			if (!(pwd = getpwnam(name))) {
 			    if (ssl_debug_flag)
 				ssl_log_msgn(bio_err,
@@ -1116,10 +1164,10 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
 		    break;
 		}
 		case 'p': { /* send the cert to the external program */
-		    cp_subject = cp_subject+2;
+		    cp_subject = cp_subject + 2;
 
 		    ret = !ssl_check_ext_prog(cert, name, strlen(name),
-			    cp_subject);
+					    cp_subject);
 		    break;
 		}
 		default:
@@ -1143,10 +1191,10 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
 	    case 1: /* allow */
 		return 1;
 	    default:
-		/* programming error */
+		/* Programming error */
 		ssl_log_msgn(bio_err,
-		"ssl_x509_auth(): internal: unhandled action flag \"%d\" (action \"%s\")",
-	    		     action_flag, cp_action);
+"ssl_x509_auth(): internal: unhandled action flag '%d' (action '%s')",
+		    action_flag, cp_action);
 	    }
 	}
     }
@@ -1158,8 +1206,9 @@ ssl_x509_auth(char *service, X509 *cert, char *name)
 
 #else /* !USE_SSL */
 
-/* something here to stop warnings if we build without TLS/SSL support */
-static int dummy_func()
+/* Something here to stop warnings if we build without TLS/SSL support */
+static int
+dummy_func()
 {
     return 0;
 }
